@@ -14,6 +14,7 @@ package geogebra.export;
 
 import geogebra.euclidian.EuclidianView;
 import geogebra.gui.TitlePanel;
+import geogebra.gui.view.Gridable;
 import geogebra.main.Application;
 import geogebra.main.GeoGebraPreferences;
 
@@ -70,18 +71,27 @@ public class PrintPreview extends JDialog {
 		BufferedImage img = new BufferedImage(5, 5, BufferedImage.TYPE_INT_RGB);
 		tempGraphics = img.getGraphics(); 
 	}		
+	
+	public PrintPreview(Application app,Gridable target){
+		this(app,target,PageFormat.PORTRAIT);
+	}
 
 	public PrintPreview(Application app, Printable target) {
 		this(app, target, PageFormat.PORTRAIT);		
 	}
 
 	public PrintPreview(Application app, Printable target, int orientation) {
-		super(app.getFrame(), true);
+		super(app.getFrame(), true); //modal=true: user shouldn't be able to change anything before actual print happened.
 		this.app = app;
 		initPrintPreview(target, orientation);
+		setDefaultCloseOperation(EXIT_ON_CLOSE);
 	}
 		
-	private void initPrintPreview(Printable target, int orientation) {	
+	public PrintPreview(Application app, Gridable target, int portrait) {
+		this(app,new PrintGridable(target),portrait);
+	}
+
+	private void initPrintPreview(Printable target, int orientation) {
 		m_target = target;
 		m_orientation = orientation;
 		m_scale = 75; // init scale to 75%		
@@ -223,8 +233,9 @@ public class PrintPreview extends JDialog {
 		
 		// show scale panel for euclidian view
 		EuclidianView ev = app.getEuclidianView();
+		EuclidianView ev2 = app.getEuclidianView2();
 		app.clearSelectedGeos();
-		if (m_target == ev) {		
+		if (m_target == ev || m_target == ev2) {		
 			// checkbox to turn on/off printing of scale string
 			cbEVscalePanel = new JCheckBox();
 			cbEVscalePanel.setSelected(app.isPrintScaleString());
@@ -236,7 +247,7 @@ public class PrintPreview extends JDialog {
 			});
 						
 			// scale panel to set scale of x-axis in cm
-			PrintScalePanel scalePanel = new PrintScalePanel(app, ev);				
+			PrintScalePanel scalePanel = new PrintScalePanel(app, (EuclidianView) m_target);				
 			scalePanel.addActionListener(lst);									
 			
 			JPanel tempPanel = new JPanel();
@@ -569,7 +580,7 @@ public class PrintPreview extends JDialog {
 								
 			setBackground(Color.white);
 			setBorder(new MatteBorder(1, 1, 2, 2, Color.black));
-			update();
+//			update();
 		}		
 	
 		public void setPageFormat(PageFormat format) {
@@ -620,7 +631,13 @@ public class PrintPreview extends JDialog {
 		}
 		
 		public void update() {
-			updateBufferedImage();
+			try {
+				updateBufferedImage();
+			} catch (Exception e) {
+				e.printStackTrace();
+			} catch (OutOfMemoryError e){
+				e.printStackTrace();
+			}
 			repaint();			
 		}
 
