@@ -26,24 +26,23 @@ import java.net.URI;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.Locale;
 import java.util.StringTokenizer;
 
 public class FileDropTargetListener implements DropTargetListener {
-
-	static DataFlavor urlFlavor, uriListFlavor, macPictStreamFlavor;
+	
+	static DataFlavor urlFlavor, macPictStreamFlavor;
 	static {
 
 		try { 
 			urlFlavor = 
-			new DataFlavor ("application/x-java-url; class=java.net.URL"); 
-			uriListFlavor = 
-			new DataFlavor ("text/uri-list; class=java.lang.String");
+			new DataFlavor ("application/x-java-url; class=java.net.URL"); 			
 		} catch (ClassNotFoundException cnfe) { 
 			cnfe.printStackTrace( );
 		}
 	}
 
+
+
 	private Application app;
 
 	public FileDropTargetListener(Application app) {
@@ -72,49 +71,80 @@ public class FileDropTargetListener implements DropTargetListener {
 			return;
 		}
 
-		ArrayList al = getGGBfiles(event);	
-		
-		if (al.size() == 0) {
-			event.dropComplete(false);
-		} else if (app.isSaved() || app.saveCurrentFile()) {				
+		event.dropComplete(handleFileDrop(event.getTransferable()));
+
+	}
+
+	/**
+	 * Determines if a transferable contains ggb files and attempts
+	 * to open them.
+	 * 
+	 * @param t
+	 * @return
+	 */
+	public boolean handleFileDrop(Transferable t){
+
+		ArrayList<File> al = getGGBfiles(t);	
+
+		if (al.size() == 0 || !isGGBFile(al.get(0).getName())) {
+			return false;
+		}
+		else if (app.isSaved() || app.saveCurrentFile()) {				
 			File [] files = new File[al.size()];
 			for (int i = 0 ; i < al.size() ; i++)
-				files[i] = (File)al.get(i);
+				files[i] = al.get(i);
 			app.getGuiManager().doOpenFiles(files, true);			
-			event.dropComplete(true);			
+			return true;			
 		}			
+		return false;
 	}
 	
-	private ArrayList getGGBfiles(DropTargetDropEvent event) {
-		Transferable transferable = event.getTransferable();
+	
+	/**
+	 * Tests if a file has the GeoGebra extension
+	 * @param fileName
+	 * @return
+	 */
+	private boolean isGGBFile(String fileName){
+		int mid = fileName.lastIndexOf(".");
+	    String ext = fileName.substring(mid+1,fileName.length());
+	    return ext.equals(Application.FILE_EXT_GEOGEBRA);
+	}
 
-		ArrayList al = new ArrayList();
+	
+
+	private ArrayList<File> getGGBfiles(Transferable transferable) {
+
+		ArrayList<File> al = new ArrayList<File>();
 
 		try {
 			// try to get an image
 			if (transferable.isDataFlavorSupported (DataFlavor.imageFlavor)) { 
-				System.out.println ("image flavor not supported"); 
+				Application.debug("image flavor not supported"); 
 				//Image img = (Image) trans.getTransferData (DataFlavor.imageFlavor); 
 			} else if (transferable.isDataFlavorSupported (DataFlavor.javaFileListFlavor)) {
 				//Application.debug("javaFileList is supported");
-				List list = (List)transferable.getTransferData (DataFlavor.javaFileListFlavor);
-				ListIterator it = list.listIterator( );    
+				List<File> list = (List<File>)transferable.getTransferData (DataFlavor.javaFileListFlavor);
+				ListIterator<File> it = list.listIterator( );    
 				while (it.hasNext( )) {
 					File f = (File) it.next( );
 					al.add(f);
 				}
-			} else if (transferable.isDataFlavorSupported (uriListFlavor)) {
+			} else if (transferable.isDataFlavorSupported (GuiManager.uriListFlavor)) {
 				//Application.debug("uri-list flavor is supported"); 
 				String uris = (String)
-				transferable.getTransferData (uriListFlavor);
+				transferable.getTransferData (GuiManager.uriListFlavor);
 
 				// url-lists are defined by rfc 2483 as crlf-delimited 
 				StringTokenizer st = new StringTokenizer (uris, "\r\n");   
 				while (st.hasMoreTokens ( )) {
 					String uriString = st.nextToken( );
-					URI uri = new URI(uriString);
-					System.out.println (uri);
-					al.add(new File(uri));
+					if(uriString.startsWith("http://") && isGGBFile(uriString)){
+						app.getGuiManager().loadURL(uriString, true);
+					}else{
+						URI uri = new URI(uriString);
+						al.add(new File(uri));
+					}
 				}
 			} else if (transferable.isDataFlavorSupported (urlFlavor)) {
 				Application.debug("url flavor not supported");
@@ -123,7 +153,7 @@ public class FileDropTargetListener implements DropTargetListener {
 		} catch (Exception e) {
 			e.printStackTrace( );
 		} 
-		
+
 		return al;
 	}
 }
\ No newline at end of file
