@@ -22,7 +22,6 @@ import geogebra.GeoGebra;
 import geogebra.euclidian.EuclidianView;
 import geogebra.kernel.Construction;
 import geogebra.kernel.GeoElement;
-import geogebra.kernel.GeoImage;
 import geogebra.kernel.Kernel;
 import geogebra.kernel.Macro;
 import geogebra.main.Application;
@@ -69,15 +68,18 @@ public class MyXMLio {
 	// an entry named XML_FILE_MACRO for the macros
 	final private static String XML_FILE_MACRO = "geogebra_macro.xml";
 
+	// library javascript available to GeoJavaScriptButton objects
+	final private static String JAVASCRIPT_FILE = "geogebra_javascript.js";
+
 	// All xml output is zipped. The created zip archive *may* contain
 	// an entry named XML_FILE_THUMBNAIL for the construction
 	final private static String XML_FILE_THUMBNAIL = "geogebra_thumbnail.png";
 	// Added for Intergeo File Format (Yves Kreis) -->
 	final private static String I2G_FILE_THUMBNAIL = "construction/preview.png";
 	// <-- Added for Intergeo File Format (Yves Kreis)
-	final private static double THUMBNAIL_PIXELS_X = 200.0; // max no of
+	final public static double THUMBNAIL_PIXELS_X = 200.0; // max no of
 															// horizontal pixels
-	final private static double THUMBNAIL_PIXELS_Y = 200.0; // max no of
+	final public static double THUMBNAIL_PIXELS_Y = 200.0; // max no of
 															// vertical pixels
 
 	// Added for Intergeo File Format (Yves Kreis) -->
@@ -112,7 +114,9 @@ public class MyXMLio {
 	
 	private DocHandler getGGBHandler() {
 		if (ggbDocHandler == null)
-			ggbDocHandler = new MyXMLHandler(kernel, cons);		
+		    //ggb3D : to create also a MyXMLHandler3D
+			//ggbDocHandler = new MyXMLHandler(kernel, cons);		
+			ggbDocHandler = kernel.newMyXMLHandler(cons);
 		return ggbDocHandler;
 	}
 	
@@ -162,7 +166,7 @@ public class MyXMLio {
 		boolean xmlFound = false;
 		boolean macroXMLfound = false;
 
-		boolean ggbFound = false;
+		boolean ggbHandler = false;
 		
 		// get all entries from the zip archive
 		while (true) {
@@ -175,10 +179,10 @@ public class MyXMLio {
 				// load xml file into memory first
 				xmlFileBuffer = Util.loadIntoMemory(zip);
 				xmlFound = true;
-				ggbFound = true;
+				ggbHandler = true;
 				// Added for Intergeo File Format (Yves Kreis) -->
 				handler = getGGBHandler();
-			} else if (!ggbFound && name.equals(I2G_FILE)) {
+			} else if (!ggbHandler && name.equals(I2G_FILE)) {
 				// load i2g file into memory first
 				xmlFileBuffer = Util.loadIntoMemory(zip);
 				xmlFound = true;
@@ -188,7 +192,11 @@ public class MyXMLio {
 				// load macro xml file into memory first
 				macroXmlFileBuffer = Util.loadIntoMemory(zip);
 				macroXMLfound = true;
+				ggbHandler = true;
 				handler = getGGBHandler();
+			} else if (name.equals(JAVASCRIPT_FILE)) {
+				// load JavaScript
+				kernel.setLibraryJavaScript(Util.loadIntoString(zip));
 			} else {
 				// try to load image
 				try {
@@ -279,39 +287,46 @@ public class MyXMLio {
 		InputStreamReader ir = new InputStreamReader(bs, "UTF8");
 
 		// process xml file
-		doParseXML(ir, clearConstruction, isGGTFile);
+		doParseXML(ir, clearConstruction, isGGTFile, true,true);
 
 		ir.close();
 		bs.close();
 	}
-
+	
 	private void doParseXML(Reader ir, boolean clearConstruction,
-			boolean isGGTFile) throws Exception {
+			boolean isGGTFile, boolean mayZoom,boolean settingsBatch) throws Exception {
 		boolean oldVal = kernel.isNotifyViewsActive();
-		if (!isGGTFile) {
+		boolean oldVal2 = kernel.isUsingInternalCommandNames();
+		kernel.setUseInternalCommandNames(true);
+		
+		if (!isGGTFile && mayZoom) {
 			kernel.setNotifyViewsActive(false);
 		}
 
 		if (clearConstruction) {
-			// clear toolbar definition
-			if (app.hasGuiManager()) {
-				app.getGuiManager().setToolBarDefinition(null);
-			}
-
 			// clear construction
 			kernel.clearConstruction();
 		}
 
 		try {
-			xmlParser.parse(handler, ir);
+			kernel.setLoadingMode(true);
+			if(settingsBatch && !isGGTFile){
+				app.getSettings().beginBatch();
+				xmlParser.parse(handler, ir);
+				app.getSettings().endBatch();
+			}
+			else
+				xmlParser.parse(handler, ir);
 			xmlParser.reset();
+			kernel.setLoadingMode(false);
 		} catch (Error e) {
 			// e.printStackTrace();
 			throw e;
 		} catch (Exception e) {
 			throw e;
 		} finally {
-			if (!isGGTFile) {
+			kernel.setUseInternalCommandNames(oldVal2);
+			if (!isGGTFile && mayZoom) {
 				kernel.updateConstruction();
 				kernel.setNotifyViewsActive(oldVal);				
 			}
@@ -322,7 +337,15 @@ public class MyXMLio {
 		if (!isGGTFile && oldVal &&
 				app.showConsProtNavigation()) 
 		{
-				app.getGuiManager().setConstructionStep(handler.getConsStep());
+				//app.getGuiManager().setConstructionStep(handler.getConsStep());
+
+			if (app.getGuiManager() != null)
+				// if there is a ConstructionProtocolView, then update its navigation bars
+				app.getGuiManager().getConstructionProtocolView().setConstructionStep(handler.getConsStep());
+			else
+				// otherwise this is not needed 
+				app.getKernel().getConstruction().setStep(handler.getConsStep());
+				
 		}
 		
 	}
@@ -338,7 +361,7 @@ public class MyXMLio {
 		ZipEntry entry = zip.getNextEntry();
 		if (entry != null && entry.getName().equals(XML_FILE)) {
 			// process xml file
-			doParseXML(new InputStreamReader(zip, "UTF8"), true, false);
+			doParseXML(new InputStreamReader(zip, "UTF8"), true, false, true,true);
 			zip.close();
 		} else {
 			zip.close();
@@ -347,11 +370,13 @@ public class MyXMLio {
 		
 		System.gc();
 	}
-
-	public void processXMLString(String str, boolean clearAll, boolean isGGTfile)
+	public void processXMLString(String str, boolean clearAll, boolean isGGTfile) throws Exception{
+		processXMLString(str,clearAll,isGGTfile,true);
+	}
+	public void processXMLString(String str, boolean clearAll, boolean isGGTfile, boolean settingsBatch)
 			throws Exception {
 		StringReader rs = new StringReader(str);
-		doParseXML(rs, clearAll, isGGTfile);
+		doParseXML(rs, clearAll, isGGTfile, clearAll, settingsBatch);
 		rs.close();
 	}
 
@@ -368,7 +393,7 @@ public class MyXMLio {
 		if (Application.getExtension(file)
 				.equals(Application.FILE_EXT_INTERGEO)) {
 			// File Extension for Intergeo: I2G
-			writeIntergeoFile(b);
+			writeIntergeoFile(b, true);
 		} else {
 			// File Extension for GeoGebra: GGB or GGT
 			writeGeoGebraFile(b, true);
@@ -382,9 +407,10 @@ public class MyXMLio {
 	 * Creates a zipped file containing the construction and all settings saved
 	 * in xml format plus all external images. GeoGebra File Format.
 	 */
-	public void writeGeoGebraFile(OutputStream os, boolean includeThumbnail) throws IOException {
+	public void writeGeoGebraFile(OutputStream os, boolean includeThumbail) throws IOException {
 		boolean isSaving = kernel.isSaving();
 		kernel.setSaving(true);
+		
 		try {
 			// zip stream
 			ZipOutputStream zip = new ZipOutputStream(os);
@@ -396,7 +422,7 @@ public class MyXMLio {
 			// Modified for Intergeo File Format (Yves Kreis) -->
 			// write construction thumbnails
 			// writeThumbnail(kernel.getConstruction(), zip);
-			if (includeThumbnail)
+			if (includeThumbail)
 				writeThumbnail(kernel.getConstruction(), zip, XML_FILE_THUMBNAIL);
 			// <-- Modified for Intergeo File Format (Yves Kreis)
 	
@@ -415,6 +441,12 @@ public class MyXMLio {
 				zip.closeEntry();
 			}
 	
+			// write library JavaScript to one special file in zip
+			zip.putNextEntry(new ZipEntry(JAVASCRIPT_FILE));
+			osw.write(kernel.getLibraryJavaScript());
+			osw.flush();
+			zip.closeEntry();
+	
 			// write XML file for construction
 			zip.putNextEntry(new ZipEntry(XML_FILE));
 			osw.write(getFullXML());
@@ -436,7 +468,7 @@ public class MyXMLio {
 	 * Creates a zipped file containing the construction saved in xml and i2g
 	 * format plus all external images. Intergeo File Format (Yves Kreis)
 	 */
-	public void writeIntergeoFile(OutputStream os) throws IOException {
+	public void writeIntergeoFile(OutputStream os, boolean includeThumbail) throws IOException {
 		boolean isSaving = kernel.isSaving();
 		kernel.setSaving(true);
 		
@@ -452,11 +484,11 @@ public class MyXMLio {
 			zip.closeEntry();
 	
 			// write construction thumbnails
-			writeThumbnail(kernel.getConstruction(), zip, I2G_FILE_THUMBNAIL);
+			if (includeThumbail)
+				writeThumbnail(kernel.getConstruction(), zip, I2G_FILE_THUMBNAIL);
 	
 			// write construction images
-			writeConstructionImages(kernel.getConstruction(), zip,
-					I2G_IMAGES);
+			writeConstructionImages(kernel.getConstruction(), zip, I2G_IMAGES);
 	
 			// save macros
 			if (kernel.hasMacros()) {
@@ -473,6 +505,12 @@ public class MyXMLio {
 				zip.closeEntry();
 			}
 	
+			// write library JavaScript to one special file in zip
+			zip.putNextEntry(new ZipEntry(I2G_PRIVATE + JAVASCRIPT_FILE));
+			osw.write(kernel.getLibraryJavaScript());
+			osw.flush();
+			zip.closeEntry();
+	
 			// write XML file for construction
 			zip.putNextEntry(new ZipEntry(I2G_PRIVATE + XML_FILE));
 			osw.write(getFullXML());
@@ -542,22 +580,22 @@ public class MyXMLio {
 			ZipOutputStream zip, String filePath) throws IOException {
 		// <-- Modified for Intergeo File Format (Yves Kreis)
 		// save all GeoImage images
-		TreeSet images = cons.getGeoSetLabelOrder(GeoElement.GEO_CLASS_IMAGE);
-		if (images == null)
+		//TreeSet images = cons.getGeoSetLabelOrder(GeoElement.GEO_CLASS_IMAGE);
+		TreeSet geos = cons.getGeoSetLabelOrder();
+		if (geos == null)
 			return;
 
-		Iterator it = images.iterator();
+		Iterator<GeoElement> it = geos.iterator();
 		while (it.hasNext()) {
-			GeoImage geoImage = (GeoImage) it.next();
+			GeoElement geo =  it.next();
 			// Michael Borcherds 2007-12-10 this line put back (not needed now
 			// MD5 code put in the correct place!)
-			String fileName = geoImage.getFileName();
-			BufferedImage img = geoImage.getImage();
-			if (img != null)
-				// Modified for Intergeo File Format (Yves Kreis) -->
-				// writeImageToZip(zip, fileName, img);
-				writeImageToZip(zip, filePath + fileName, img);
-			// <-- Modified for Intergeo File Format (Yves Kreis)
+			String fileName = geo.getImageFileName();
+			if (fileName != null) {
+				BufferedImage img = geo.getFillImage();
+				if (img != null)
+					writeImageToZip(zip, filePath + fileName, img);
+			}
 		}
 	}
 
@@ -574,11 +612,15 @@ public class MyXMLio {
 		EuclidianView ev = app.getEuclidianView();
 
 		// max 128 pixels either way
+		/*
 		double exportScale = Math.min(THUMBNAIL_PIXELS_X
-				/ ev.getSelectedWidth(), THUMBNAIL_PIXELS_Y
+				/ ev.getSelectedWidth(), THUMBNAIL_PIXELS_X
 				/ ev.getSelectedHeight());
+		*/
+				 
 		try {
-			BufferedImage img = app.getEuclidianView().getExportImage(exportScale);
+			//BufferedImage img = app.getExportImage(exportScale);
+			BufferedImage img = app.getExportImage(THUMBNAIL_PIXELS_X,THUMBNAIL_PIXELS_Y);
 			if (img != null)
 				// Modified for Intergeo File Format (Yves Kreis) -->
 				// writeImageToZip(zip, XML_FILE_THUMBNAIL, img);
@@ -647,7 +689,20 @@ public class MyXMLio {
 				ext = "JPG";
 			else
 				ext = "PNG";
+			
+			boolean useCache = true;
+			// circumvent security issues by disabling disk-based caching
+			if(app.isApplet()) {
+				useCache = ImageIO.getUseCache();
+				javax.imageio.ImageIO.setUseCache(false);
+			}
+			
 			ImageIO.write(img, ext, os);
+			
+			// restore caching to prevent side-effects
+			if(app.isApplet()) {
+				javax.imageio.ImageIO.setUseCache(useCache);
+			}
 		} catch (Exception e) {
 			Application.debug(e.getMessage());
 			try {
@@ -663,7 +718,7 @@ public class MyXMLio {
 	/**
 	 * Compresses xml String and writes result to os.
 	 */
-	public static void writeZipped(OutputStream os, StringBuffer xmlString) throws IOException {
+	public static void writeZipped(OutputStream os, StringBuilder xmlString) throws IOException {
 		ZipOutputStream z = new ZipOutputStream(os);
 		z.putNextEntry(new ZipEntry(XML_FILE));
 		BufferedWriter w = new BufferedWriter(new OutputStreamWriter(z, "UTF8"));
@@ -673,32 +728,60 @@ public class MyXMLio {
 		w.close();
 		z.close();
 	}
+	
+	private final static void addXMLHeader(StringBuilder sb) {
+		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+	}
+
+	private final static void addGeoGebraHeader(StringBuilder sb, boolean isMacro, String uniqueId) {
+		sb.append("<geogebra format=\"");
+		sb.append(GeoGebra.XML_FILE_FORMAT);
+		sb.append("\" ");
+		if (uniqueId != null) {
+			sb.append("id=\"");
+			sb.append(uniqueId); // unique id to identify ggb file
+			sb.append("\" ");
+		}
+		sb.append(" xsi:noNamespaceSchemaLocation=\"http://www.geogebra.org/");
+		if (isMacro)
+			sb.append(GeoGebra.GGT_XSD_FILENAME); //eg	ggt.xsd
+		else
+			sb.append(GeoGebra.GGB_XSD_FILENAME); //eg	ggb.xsd
+		sb.append("\" xmlns=\"\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" >\n");
+	}
+	
 
 	/**
 	 * Returns XML representation of all settings and construction. GeoGebra
 	 * File Format.
 	 */
 	public String getFullXML() {
-		StringBuffer sb = new StringBuffer();
-		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-		sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT + "\">\n");
+		StringBuilder sb = new StringBuilder();
+		addXMLHeader(sb);
+		addGeoGebraHeader(sb, false, app.getUniqueId());
+		//sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+		//sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT + "\"");
+		//sb.append(" xsi:noNamespaceSchemaLocation=\"http://www.geogebra.org/");
+		//sb.append(GeoGebra.GGB_XSD_FILENAME); //eg	ggb.xsd
+		//sb.append("\" xmlns=\"\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" >\n");
 
 		// save gui settings
-		sb.append(app.getCompleteUserInterfaceXML());		
+		sb.append(app.getCompleteUserInterfaceXML(false));		
 
 		// save construction
-		sb.append(cons.getConstructionXML());
-
+		cons.getConstructionXML(sb);
+		
 		sb.append("</geogebra>");
 		return sb.toString();
 	}
 
 	/**
-	 * Returns I2G representation of construction. Intergeo File Format. (Yves
-	 * Kreis)
+	 * Returns I2G representation of construction. Intergeo File Format.
+	 * (Yves Kreis)
 	 */
 	public String getFullI2G() {
-		StringBuffer sb = new StringBuffer();
+		StringBuilder sb = new StringBuilder();
+		//addXMLHeader(sb);
 
 		sb.append("<!--\n\tIntergeo File Format Version "
 				+ GeoGebra.I2G_FILE_FORMAT + "\n\twritten by "
@@ -709,10 +792,10 @@ public class MyXMLio {
 		sb.append("<construction>\n");
 
 		// save construction
-		sb.append(kernel.getConstructionI2G());
+		cons.getConstructionI2G(sb, Construction.CONSTRUCTION);
 
-		StringBuffer display = new StringBuffer();
-		display.append(kernel.getDisplayI2G());
+		StringBuilder display = new StringBuilder();
+		cons.getConstructionI2G(display, Construction.DISPLAY);
 		if (!display.toString().equals("")) {
 			sb.append("\t<display>\n");
 			sb.append(display.toString());
@@ -728,14 +811,16 @@ public class MyXMLio {
 	 * Returns XML representation of all settings WITHOUT construction.
 	 */
 	public String getPreferencesXML() {
-		StringBuffer sb = new StringBuffer();
-		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-		sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT
-						+ "\">\n");
+		StringBuilder sb = new StringBuilder();
+		addXMLHeader(sb);
+		addGeoGebraHeader(sb, false, null);
+		//sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+		//sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT
+		//				+ "\">\n");
 
 		// save gui settings
-		sb.append(app.getCompleteUserInterfaceXML());		
-
+		sb.append(app.getCompleteUserInterfaceXML(true));
+		
 		sb.append("</geogebra>");
 		return sb.toString();
 	}
@@ -744,10 +829,12 @@ public class MyXMLio {
 	 * Returns XML representation of given macros in the kernel.
 	 */
 	public String getFullMacroXML(ArrayList macros) {
-		StringBuffer sb = new StringBuffer();
-		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-		sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT
-						+ "\">\n");
+		StringBuilder sb = new StringBuilder();
+		addXMLHeader(sb);
+		addGeoGebraHeader(sb, true, null);
+		//sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+		//sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT
+		//				+ "\">\n");
 
 		// save construction
 		sb.append(kernel.getMacroXML(macros));
@@ -760,29 +847,46 @@ public class MyXMLio {
 	 * Returns XML representation of all settings and construction needed for
 	 * undo.
 	 */
-	public static synchronized StringBuffer getUndoXML(Construction c) {
+	public static synchronized StringBuilder getUndoXML(Construction c) {
 		Application app = c.getApplication();
 
-		StringBuffer sb = new StringBuffer();
-		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-		sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT + "\">\n");
+		StringBuilder sb = new StringBuilder();
+		addXMLHeader(sb);
+		addGeoGebraHeader(sb, false, app.getUniqueId());
+		//sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+		//sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT + "\">\n");
 
 		// save euclidianView settings
-		String addStr = app.getEuclidianView().getXML();
-		sb.ensureCapacity(sb.length() + addStr.length());
-		sb.append(addStr);
+		app.getEuclidianViewXML(sb,false);
+		
+		//save euclidian view 2 settings
+		if(app.hasEuclidianView2EitherShowingOrNot()){
+			app.getEuclidianView2().getXML(sb,false);
+		}
 		
 		// save kernel settings
-		addStr = c.getKernel().getKernelXML();
-		sb.ensureCapacity(sb.length() + addStr.length());
-		sb.append(addStr);
+		c.getKernel().getKernelXML(sb, false);
+
+		// save construction
+		c.getConstructionXML(sb);
+
+//		// save cas session
+//		if (app.useFullGui() && app.getGuiManager().hasCasView()) {
+//			app.getGuiManager().getCasView().getSessionXML(sb);
+//		}
+		
+		// save spreadsheetView settings
+	//	if (app.useFullGui() && app.getGuiManager().hasSpreadsheetView())
+	//		app.getGuiManager().getSpreadsheetViewXML(sb);
 		
-		// save construction
-		addStr = c.getConstructionXML();
-		sb.ensureCapacity(sb.length() + addStr.length());
-		sb.append(addStr);
+		// save algebraView settings
+		//app.getGuiManager().getAlgebraViewXML(sb);
+		
+		// save ProbabilityCalculator settings
+		if (app.useFullGui() && app.getGuiManager().hasProbabilityCalculator()){
+			app.getGuiManager().getProbabilityCalculatorXML(sb);
+		}
 
-		sb.ensureCapacity(sb.length() + 20);
 		sb.append("</geogebra>");
 
 		/*
@@ -800,4 +904,15 @@ public class MyXMLio {
 	 * ImageIO.getWriterFormatNames(); for (int i=0; i < formats.length; i++) {
 	 * Application.debug(formats[i]); } }
 	 */
+
+	/** 
+	* Returns .out representation for regression testing. 
+	*/ 
+	public String getConstructionRegressionOut() {
+		StringBuilder sb = new StringBuilder();
+		cons.getConstructionRegressionOut(sb);
+		return sb.toString();
+	}		         
+
 }
+
