@@ -18,6 +18,8 @@ the Free Software Foundation.
 
 package geogebra.kernel;
 
+import geogebra.euclidian.EuclidianConstants;
+
 import java.util.ArrayList;
 import java.util.Arrays;
 
@@ -56,15 +58,28 @@ public class AlgoIntersectConics extends AlgoIntersect {
     private PointPairList pointList = new PointPairList();
     
     private EquationSolver eqnSolver;
+    private SystemOfEquationsSolver sysSolver;
         
-	protected String getClassName() {
+	public String getClassName() {
 		return "AlgoIntersectConics";
 	}
+	
+	public int getRelatedModeID() {
+    	return EuclidianConstants.MODE_INTERSECT;
+    }
+    
+	public AlgoIntersectConics(Construction cons) {           
+    	super(cons); 
+    	
+		eqnSolver = cons.getEquationSolver();
+		sysSolver = new SystemOfEquationsSolver(eqnSolver);
+		
+		degConic = new GeoConic(cons);  
+	       
+	}
     
     AlgoIntersectConics(Construction cons, GeoConic A, GeoConic B) {           
-    	super(cons);     
-    	
-		eqnSolver = cons.getEquationSolver();
+    	this(cons);     
     	
         this.A = A;
         this.B = B;   
@@ -86,19 +101,26 @@ public class AlgoIntersectConics extends AlgoIntersect {
             D[i] = new GeoPoint(cons);            
         }                                   
         
-        //degConic = new GeoConic[3];
-        //for (i=0; i < 3; i++) degConic[i] = new GeoConic(cons);
-        degConic = new GeoConic(cons);  
+
         
         // check possible special case
         possibleSpecialCase = handleSpecialCase();
         
         setInputOutput(); // for AlgoElement     
         initForNearToRelationship();
-        compute();                      
+        compute(); 
+        setIncidence();
+
     }
     
-    // for AlgoElement
+    private void setIncidence() {
+        for (i=0; i < 4; i++) {
+        	P[i].addIncidence(A);
+    		P[i].addIncidence(B);
+        }
+	}
+
+	// for AlgoElement
     public void setInputOutput() {
         input = new GeoElement[2];
         input[0] = A;
@@ -109,14 +131,14 @@ public class AlgoIntersectConics extends AlgoIntersect {
         setDependencies(); // done by AlgoElement
     }    
         
-	GeoPoint [] getIntersectionPoints() {
+	protected GeoPoint [] getIntersectionPoints() {
 		return P;
 	}
 	
     GeoConic getA() { return A; }
     GeoConic getB() { return B; }
 	
-	GeoPoint [] getLastDefinedIntersectionPoints() {
+	protected GeoPoint [] getLastDefinedIntersectionPoints() {
 		return D;
 	}
 	
@@ -124,7 +146,7 @@ public class AlgoIntersectConics extends AlgoIntersect {
     	return true;
     }
 	
-	final void initForNearToRelationship() {     
+	protected final void initForNearToRelationship() {     
 		isPermutationNeeded = true;
     	for (int i=0; i < P.length; i++) {        	 	 
     	 	 age[i] = 0; 
@@ -146,7 +168,7 @@ public class AlgoIntersectConics extends AlgoIntersect {
     	// check for special case of two circles with common point
         if (possibleSpecialCase) {
             if (handleSpecialCase()) return;
-        }
+        }     	
     	
         // continous: use near-to-heuristic between old and new intersection points
         // non-continous: use computeContinous() to init a permutation and then
@@ -180,6 +202,7 @@ public class AlgoIntersectConics extends AlgoIntersect {
     	if (pointOnConic == null) 
     		// check if we have a point on B that is also on A
     		 pointOnConic = getPointFrom1on2(B, A); 
+    	
     	// if we didn't have a common point, there's no special case
     	if (pointOnConic == null) 
     		return false;				    	
@@ -243,8 +266,17 @@ public class AlgoIntersectConics extends AlgoIntersect {
 			int size = pointsOnConic.size();
 			for (int i=0; i < size; i++) {
 				GeoPoint p = (GeoPoint) pointsOnConic.get(i);
-				if (B.isOnPath(p, Kernel.MIN_PRECISION)) {
-					pointOnConic = p;
+				//if (B.isOnPath(p, Kernel.MIN_PRECISION)) {
+				if (p.isLabelSet() && 
+						p.getIncidenceList()!=null && 
+						p.getIncidenceList().contains(B)) {
+					
+					//TODO: a potential temporary fix for #94.
+    				if (A.isOnPath(p, Kernel.EPSILON) && B.isOnPath(p, Kernel.EPSILON))
+    					pointOnConic = p;
+    				
+					
+					//pointOnConic = p;
 					break;
 				}
 			}
@@ -421,7 +453,7 @@ public class AlgoIntersectConics extends AlgoIntersect {
     
     // calc four intersection Points of conics A and B.
     // write result into points
-    final void intersectConics(GeoConic conic1, GeoConic conic2, 
+    final public void intersectConics(GeoConic conic1, GeoConic conic2, 
                                         GeoPoint[] points) {
     	    	 
     	if (!(conic1.isDefined() && conic2.isDefined())) {
@@ -437,7 +469,7 @@ public class AlgoIntersectConics extends AlgoIntersect {
         if (conic1.equals(conic2)) {
             for (i=0; i < points.length; i++) {
                 points[i].setUndefined();
-            }
+            }            
             return;
         } 
     	
@@ -457,8 +489,8 @@ public class AlgoIntersectConics extends AlgoIntersect {
             kernel.setEpsilon(epsilon);                        
             
             // find intersection points conics through intersection points
-        	ok = calcIntersectionPoints(conic1, conic2, points, epsilon); 
-        	                            
+        	ok = calcIntersectionPoints(conic1, conic2, points, epsilon);  
+                                         
             // try it with lower precision     
             epsilon *= 10.0;                        
         }            
@@ -742,7 +774,41 @@ public class AlgoIntersectConics extends AlgoIntersect {
         	if (testPoints(A, B, points, Kernel.MIN_PRECISION))
         		return true;			   	               
  	   }
-            
+ 	   
+ 	    // If intersection points not found
+ 	    // try with another algorithm - solving system of algebraic equations of conics
+ 	    /* Author ddrakulic */
+ 	    
+ 	    double [] param1 = new double[6];
+ 	    param1[0] = A.matrix[0]; //x^2
+ 	    param1[1] = 2*A.matrix[3]; //xy
+ 	    param1[2] = A.matrix[1]; //y^2
+ 	    param1[3] = 2*A.matrix[4]; //x
+	    param1[4] = 2*A.matrix[5]; //y
+	    param1[5] = A.matrix[2]; //constant
+
+ 	    double [] param2 = new double[6];
+ 	    param2[0] = B.matrix[0]; //x^2
+ 	    param2[1] = 2*B.matrix[3]; //xy
+ 	    param2[2] = B.matrix[1]; //y^2
+ 	    param2[3] = 2*B.matrix[4]; //x
+	    param2[4] = 2*B.matrix[5]; //y
+	    param2[5] = B.matrix[2]; //constant
+	    
+	    double [][] res = new double[4][2];
+	    
+	    // Solving system of equations 
+	    solnr = sysSolver.solveSystemOfQuadraticEquations(param1, param2, res);
+	    
+	    for(int i=0; i<solnr; i++)
+	    	points[i].setCoords(res[i][0], res[i][1], 1.0d);
+	    	
+	    for(i=solnr; i<4; i++)
+	    	points[i].setUndefined();
+	    
+	    if (testPoints(A, B, points, Kernel.MIN_PRECISION))
+    		return true;
+		
         //Application.debug("no solutions found");
         //degConic.setUndefined();
         return false;
