@@ -13,6 +13,7 @@ the Free Software Foundation.
 package geogebra.kernel;
 
 
+import geogebra.euclidian.EuclidianConstants;
 import geogebra.main.Application;
 
 import java.awt.geom.Point2D;
@@ -24,7 +25,7 @@ import java.util.TreeSet;
 /** 
  * locus line for Q dependent on P
  */
-public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
+public class AlgoLocus extends AlgoElement implements EuclidianViewCE {
 	
 	// TODO: update locus algorithm
 	// * locus of Q=(x(B), a) with a= integral[f(x), 0, x(B)] and B is point on x-axis  freezes GeoGebra
@@ -32,7 +33,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 	//
 		
 	
-	// maximum time for the computation of one locus point in millis
+	/** maximum time for the computation of one locus point in millis **/
 	public static int MAX_TIME_FOR_ONE_STEP = 500;
 	
 	private static final long serialVersionUID = 1L;
@@ -64,7 +65,8 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     private MacroKernel macroKernel;
     //private AlgorithmSet macroConsAlgoSet;
 	// list with all original elements used for the macro construction
-    private TreeSet locusConsOrigElements, Qin; 
+    private TreeSet<ConstructionElement> locusConsOrigElements;
+    private TreeSet<GeoElement> Qin; 
     
     private long countUpdates =0;
     
@@ -85,7 +87,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
         updateScreenBorders();
         locus = new GeoLocus(cons);                      
         setInputOutput(); // for AlgoElement       
-        cons.registerEuclidianViewAlgo(this);
+        cons.registerEuclidianViewCE(this);
         compute();
         
      	// we may have created a starting point for the path now
@@ -109,14 +111,23 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 //		}
 //    }
  
-    protected String getClassName() {
+    public String getClassName() {
         return "AlgoLocus";
     }
     
+    public int getRelatedModeID() {
+    	return EuclidianConstants.MODE_LOCUS;
+    }
+    
+    
     public ArrayList getMoveableInputPoints() {
     	return null;
     }
     
+    /**
+     * Returns the dependent point
+     * @return dependent point Q
+     */
     public GeoPoint getQ() {
     	return locusPoint;
     }
@@ -126,14 +137,15 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	Qin = locusPoint.getAllPredecessors(); // all parents of Q
     	
     	// get intersection of all children of P and all parents of Q    	       	
-    	locusConsOrigElements = new TreeSet(); 
-    	Iterator it = Qin.iterator();
+    	locusConsOrigElements = new TreeSet<ConstructionElement>();
+    	TreeSet<Long> usedAlgoIds = new TreeSet<Long>();
+    	Iterator<GeoElement> it = Qin.iterator();
     	while (it.hasNext()) {
-    		GeoElement parent = (GeoElement) it.next();
+    		GeoElement parent = it.next();
     		    	        		
     		if (parent.isLabelSet() && parent.isChildOf(movingPoint)) {
     			// note: locusConsOrigElements will contain AlgoElement and GeoElement objects
-    			Macro.addDependentElement(parent, locusConsOrigElements);   
+    			Macro.addDependentElement(parent, locusConsOrigElements,usedAlgoIds);   
     		}
     	}    
     	
@@ -154,7 +166,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	locusConsOrigElements.add(movingPoint);
 
     	// add locus creating point and its algorithm to locusConsOrigElements 
-		Macro.addDependentElement(locusPoint, locusConsOrigElements); 	   	
+		Macro.addDependentElement(locusPoint, locusConsOrigElements,usedAlgoIds); 	   	
     	    	      
     	// create macro construction
     	buildLocusMacroConstruction(locusConsOrigElements);
@@ -169,14 +181,14 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	// it is inefficient to have Q and P as input
     	// let's take all independent parents of Q
     	// and the path as input
-    	TreeSet inSet = new TreeSet();
+    	TreeSet<GeoElement> inSet = new TreeSet<GeoElement>();
     	inSet.add(path.toGeoElement());
     	
     	// we need all independent parents of Q PLUS
     	// all parents of Q that are points on a path    	
-    	Iterator it = Qin.iterator();
+    	Iterator<GeoElement> it = Qin.iterator();
     	while (it.hasNext()) {
-    		GeoElement geo = (GeoElement) it.next();
+    		GeoElement geo = it.next();
     		if (geo.isIndependent() || geo.isPointOnPath()) {
     			inSet.add(geo);    			
     		}
@@ -198,8 +210,8 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	standardInput[0] = locusPoint;
     	standardInput[1] = movingPoint;    	
     	
-        output = new GeoElement[1];
-        output[0] = locus;
+        setOutputLength(1);
+        setOutput(0,locus);
         
         // handle dependencies
         setEfficientDependencies(standardInput, efficientInput);           
@@ -209,12 +221,16 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
         return getCommandDescription();        
     }
 
+    /**
+     * Returns locus
+     * @return locus 
+     */
     GeoLocus getLocus() {
         return locus;
     }    
     
    
-    private void buildLocusMacroConstruction(TreeSet locusConsElements) {       	
+    private void buildLocusMacroConstruction(TreeSet<ConstructionElement> locusConsElements) {       	
     	// build macro construction
     	macroKernel = new MacroKernel(kernel); 
     	macroKernel.setGlobalVariableLookup(true);
@@ -222,9 +238,9 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	// tell the macro construction about reserved names:
     	// these names will not be looked up in the parent
     	// construction
-    	Iterator it = locusConsElements.iterator();
+    	Iterator<ConstructionElement> it = locusConsElements.iterator();
     	while (it.hasNext()) {
-    		ConstructionElement ce = (ConstructionElement) it.next();
+    		ConstructionElement ce = it.next();
     		if (ce.isGeoElement()) {
     			GeoElement geo = (GeoElement) ce;
     			macroKernel.addReservedLabel(geo.getLabel()); 
@@ -274,9 +290,9 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
      *  to the current values of the main construction    
      */
     private void resetMacroConstruction() {       
-      	Iterator it = locusConsOrigElements.iterator();
+      	Iterator<ConstructionElement> it = locusConsOrigElements.iterator();
       	while (it.hasNext()) {
-      		ConstructionElement ce = (ConstructionElement) it.next();
+      		ConstructionElement ce = it.next();
       		if (ce.isGeoElement()) {
 	      		GeoElement geoOrig = (GeoElement) ce;  
 	      		// do not copy functions, their expressions already
@@ -293,21 +309,23 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 		      		}
 	      		}
       		}
-      	}               	      
-      }             
+      	}
+      }
 
     // compute locus line
-    final protected void compute() {    	    
-    	if (!movingPoint.isDefined() || macroCons == null) {    		
+    final protected void compute() {
+    	if (!movingPoint.isDefined() || macroCons == null || !path.toGeoElement().isDefined()) {
     		locus.setUndefined();
     		return;
     	}
- 
+
     	locus.clearPoints();    
     	clearCache();
     	pointCount = 0;	 	     	
     	useCache = 0;
     	countUpdates = 0;
+    	lastX = Double.MAX_VALUE;
+    	lastY = Double.MAX_VALUE;
     	maxTimeExceeded = false;
     	foundDefined = false;
     	boolean prevQcopyDefined = false;
@@ -337,7 +355,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	    	    
     	// update Pcopy to compute Qcopy      	
     	pcopyUpdateCascade();    	    	    
-   		prevQcopyDefined = Qcopy.isDefined();   	
+   		prevQcopyDefined = Qcopy.isDefined() && !Qcopy.isInfinite();   	
     	
     	// move Pcopy along the path
     	// do this until Qcopy comes back to its start position
@@ -365,11 +383,11 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 	        	pcopyUpdateCascade();
 	        	
 	       		// Qcopy DEFINED
-	       		if (Qcopy.isDefined()) {		       				       			
+	       		if (Qcopy.isDefined() && !Qcopy.isInfinite()) {		       				       			
 	       			// STANDARD CASE: no parameter jump
 	       			if (!parameterJump) {	       					   				       			      		
 	   					// make steps smaller until distance ok to connect with last point
-	       				while (Qcopy.isDefined() && !distanceOK(Qcopy) && !maxTimeExceeded)
+	       				while (Qcopy.isDefined() && !Qcopy.isInfinite() && !distanceOK(Qcopy) && !maxTimeExceeded)
 	       				{			
 	       					//go back and try smaller step	  
 	       		        	boolean smallerStep = pathMover.smallerStep();	   
@@ -383,7 +401,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 	       		        	pcopyUpdateCascade();	               		
 	       				}	       					       					       			
 	       				
-	       				if (Qcopy.isDefined()) {
+	       				if (Qcopy.isDefined() && !Qcopy.isInfinite()) {
 	       					// draw point
 	       					insertPoint(Qcopy.inhomX, Qcopy.inhomY, distanceSmall(Qcopy));	
 	       					prevQcopyDefined = true;
@@ -446,7 +464,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 	        else {	        
 		        // make sure that Pcopy is back at startPos now
 		        // look at Qcopy at startPos	    	         
-				Pcopy.set(PstartPos);
+				Pcopy.set((GeoElement) PstartPos);
 				pcopyUpdateCascade();
 				if (Qcopy.inhomX != lastX || Qcopy.inhomY != lastY)
 					insertPoint(Qcopy.inhomX, Qcopy.inhomY, distanceSmall(Qcopy));				
@@ -530,7 +548,7 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 	    	}    	   
     	}	    	    
 
-//    	if (Qcopy.isDefined()) {
+//    	if (Qcopy.isDefined() && !Qcopy.isInfinite()) {
 //    		if (!foundDefined)
 //    			System.out.print(locus.label + " FIRST DEFINED param: " + Pcopy.getPathParameter().t);
 //    		else
@@ -542,10 +560,10 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 //    	}
       	
     	// check found defined
-    	if (!foundDefined && Qcopy.isDefined()) {
+    	if (!foundDefined && Qcopy.isDefined() && !Qcopy.isInfinite()) {
     		pathMover.init(Pcopy);
-    		PstartPos.set(Pcopy);
-        	QstartPos.set(Qcopy);
+    		PstartPos.set((GeoElement) Pcopy);
+        	QstartPos.set((GeoElement) Qcopy);
         	foundDefined = true;
         	
         	// insert first point
@@ -607,9 +625,20 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
     	
     	if (lastFarAway && isFarAway(Q.inhomX, Q.inhomY)) {
 			// if last point Q' was far away and Q is far away
-    		// let's check whether the line Q'Q intersects the
-    		// near to screen rectangle    	        		    	
-    		distanceOK = !nearToScreenRect.intersects(lastX, lastY, Q.inhomX, Q.inhomY);
+    		// then the distance is probably OK (return true),
+    		// so we probably don't need smaller step,
+    		// except if the rectangle of the segment Q'Q
+    		// intersects the near to screen rectangle
+    		// (it will probably not be on the screen anyway)
+    		double minX = lastX;
+    		double minY = lastY;
+    		double lengthX = Q.inhomX - lastX;
+    		double lengthY = Q.inhomY - lastY;
+    		if (Q.inhomX < minX) minX = Q.inhomX;
+    		if (Q.inhomY < minY) minY = Q.inhomY;
+    		if (lengthX < 0) lengthX = -lengthX;
+    		if (lengthY < 0) lengthY = -lengthY;
+    		distanceOK = !nearToScreenRect.intersects(minX, minY, lengthX, lengthY);
     	} else {
     		distanceOK = distanceSmall(Q);             		    		
     	}
@@ -650,9 +679,10 @@ public class AlgoLocus extends AlgoElement implements EuclidianViewAlgo {
 				  farYmax - farYmin);      	
     }
     
-    public void euclidianViewUpdate() {
+    public boolean euclidianViewUpdate() {
       	updateScreenBorders();
-  		update();    	
+  		update(); 
+  		return false;
    	}
  	
 }
