@@ -12,8 +12,11 @@ the Free Software Foundation.
 
 package geogebra.kernel;
 
+import geogebra.euclidian.EuclidianConstants;
 import geogebra.kernel.arithmetic.MyDouble;
 import geogebra.kernel.arithmetic.NumberValue;
+import geogebra.kernel.kernelND.GeoPointND;
+import geogebra.kernel.kernelND.GeoSegmentND;
 
 import java.util.ArrayList;
 
@@ -87,10 +90,15 @@ public class AlgoPolygonRegular extends AlgoElement {
         updateSegmentsAndPointsLabels(points.length);
     }   
         
-    protected String getClassName() {
+    public String getClassName() {
         return "AlgoPolygonRegular";
     }        
     
+    public int getRelatedModeID() {
+    	return EuclidianConstants.MODE_REGULAR_POLYGON;
+    }
+    
+    
     // for AlgoElement
     protected void setInputOutput() {
     	input = new GeoElement[3];
@@ -123,8 +131,8 @@ public class AlgoPolygonRegular extends AlgoElement {
     	// otherwise: points and segments are also output objects
     	else {
 	    	// size = poly + points (without A, B) + segments
-	    	GeoSegment [] segments = poly.getSegments();
-	    	GeoPoint [] points = poly.getPoints();
+	    	GeoSegmentND[] segments = poly.getSegments();
+	    	GeoPointND [] points = poly.getPoints();
 	        int size = 1 + segments.length + points.length - 2; 
 	       
 	        output = new GeoElement[size];   
@@ -132,11 +140,11 @@ public class AlgoPolygonRegular extends AlgoElement {
 	        output[k] = poly;                                  
 	              
 	        for (int i=0; i < segments.length; i++) {
-	            output[++k] = segments[i];
+	            output[++k] = (GeoElement) segments[i];
 	        }    
 	        
 	        for (int i=2; i < points.length; i++) {
-	            output[++k] = points[i];            
+	            output[++k] = (GeoElement) points[i];            
 	        }         
     	}
         
@@ -198,14 +206,15 @@ public class AlgoPolygonRegular extends AlgoElement {
     	// let's create the points by rotating A around the center point
     	for (int k=2; k < n; k++) {    		
     		// rotate point around center point
-    		points[k].set(A); 
+    		points[k].set((GeoElement) A); 
     		rotAngle.set(k * alpha);
     		points[k].rotate(rotAngle, centerPoint);      		
     	}
     	
     	// compute area of poly
     	poly.calcArea();  
-    	
+    	//update region coordinate system (Zbynek Konecny, 2010-05-17)
+    	poly.updateRegionCS();
     	// update new points and segments 
     	if (n != oldPointNumber) {
     		updateSegmentsAndPointsLabels(oldPointNumber);
@@ -219,6 +228,8 @@ public class AlgoPolygonRegular extends AlgoElement {
     	// set labels only when points have labels
 		labelPointsAndSegments = labelPointsAndSegments || A.isLabelSet() || B.isLabelSet();
 		
+		GeoSegmentND[] segments = poly.getSegments();    	           
+
 		boolean pointsSegmentsShowLabel = labelPointsAndSegments && 
 				(A.isEuclidianVisible() && A.isLabelVisible() || 
 				 B.isEuclidianVisible() && B.isLabelVisible());
@@ -234,20 +245,21 @@ public class AlgoPolygonRegular extends AlgoElement {
 		}
 		
 		// update all segments and set labels for new segments
-		GeoSegment [] segments = poly.getSegments();    	           
 		for (int i=0; i < segments.length; i++) {   
-			if (labelPointsAndSegments) {
-            	if (!segments[i].isLabelSet()) {
-            		segments[i].setLabel(null);
-            		segments[i].setAuxiliaryObject(true);
-            		segments[i].setLabelVisible(pointsSegmentsShowLabel);
+			GeoElement seg = (GeoElement) segments[i];
+			if (labelPointsAndSegments) {				
+            	if (!seg.isLabelSet()) {
+            		seg.setLabel(null);
+            		seg.setAuxiliaryObject(true);
+            		// show segment label only if label showing for first segment
+            		seg.setLabelVisible(pointsSegmentsShowLabel && segments[0].isLabelVisible());
             	} 
             	else {
-            		pointsSegmentsShowLabel = pointsSegmentsShowLabel || segments[i].isLabelVisible();
+            		pointsSegmentsShowLabel = pointsSegmentsShowLabel || seg.isLabelVisible();
             	}
 			}    			
         	
-        	segments[i].getParentAlgorithm().update(); 
+			seg.getParentAlgorithm().update(); 
         }
     }
     
@@ -287,14 +299,13 @@ public class AlgoPolygonRegular extends AlgoElement {
 			newPoint.setPointSize(A.pointSize);
 			newPoint.setEuclidianVisible(A.isEuclidianVisible() || B.isEuclidianVisible());
 			newPoint.setAuxiliaryObject(true);
-
 			GeoBoolean conditionToShow = A.getShowObjectCondition();
 			if (conditionToShow == null) conditionToShow = B.getShowObjectCondition();
 			if (conditionToShow != null) {
 				try { ((GeoElement)newPoint).setShowObjectCondition(conditionToShow);}
 				catch (Exception e) {}
 			}
-			
+
 			points[i] = newPoint;						 	        	     
 		}    
     }
