@@ -1,10 +1,11 @@
 package geogebra.kernel.commands;
 
-import geogebra.kernel.AlgoDependentLine;
+import geogebra.kernel.AlgoElement;
 import geogebra.kernel.CircularDefinitionException;
 import geogebra.kernel.Construction;
 import geogebra.kernel.GeoAngle;
 import geogebra.kernel.GeoBoolean;
+import geogebra.kernel.GeoCasCell;
 import geogebra.kernel.GeoConic;
 import geogebra.kernel.GeoElement;
 import geogebra.kernel.GeoFunction;
@@ -14,6 +15,7 @@ import geogebra.kernel.GeoList;
 import geogebra.kernel.GeoNumeric;
 import geogebra.kernel.GeoPoint;
 import geogebra.kernel.GeoText;
+import geogebra.kernel.GeoUserInputElement;
 import geogebra.kernel.GeoVec2D;
 import geogebra.kernel.GeoVec3D;
 import geogebra.kernel.GeoVector;
@@ -24,6 +26,8 @@ import geogebra.kernel.arithmetic.Equation;
 import geogebra.kernel.arithmetic.ExpressionNode;
 import geogebra.kernel.arithmetic.ExpressionValue;
 import geogebra.kernel.arithmetic.Function;
+import geogebra.kernel.arithmetic.FunctionNVar;
+import geogebra.kernel.arithmetic.FunctionVariable;
 import geogebra.kernel.arithmetic.MyDouble;
 import geogebra.kernel.arithmetic.MyList;
 import geogebra.kernel.arithmetic.MyStringBuffer;
@@ -33,19 +37,23 @@ import geogebra.kernel.arithmetic.Polynomial;
 import geogebra.kernel.arithmetic.TextValue;
 import geogebra.kernel.arithmetic.ValidExpression;
 import geogebra.kernel.arithmetic.VectorValue;
+import geogebra.kernel.cas.AlgoDependentCasCell;
+import geogebra.kernel.implicit.GeoImplicitPoly;
+import geogebra.kernel.kernelND.GeoPointND;
 import geogebra.kernel.parser.ParseException;
 import geogebra.kernel.parser.Parser;
 import geogebra.main.Application;
 import geogebra.main.MyError;
 
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Set;
+import java.util.TreeSet;
 
 public class AlgebraProcessor {
 	
-	private Kernel kernel;
+	protected Kernel kernel;
 	private Construction cons;
-	private Application app;
+	protected Application app;
 	private Parser parser;
 	protected CommandDispatcher cmdDispatcher;
 	
@@ -55,19 +63,116 @@ public class AlgebraProcessor {
 		this.kernel = kernel;
 		cons = kernel.getConstruction();
 		
-		cmdDispatcher = new CommandDispatcher(kernel);
+		cmdDispatcher = newCommandDispatcher(kernel);
 		app = kernel.getApplication();
 		parser = kernel.getParser();
 	}
 	
-	public Iterator getCmdNameIterator() {
-		return cmdDispatcher.getCmdNameIterator();
+	/**
+	 * @param kernel 
+	 * @return a new command dispatcher (used for 3D)
+	 */
+	protected CommandDispatcher newCommandDispatcher(Kernel kernel){
+		return new CommandDispatcher(kernel);
 	}
 	
+	
+	
+	public Set getPublicCommandSet() {
+		return cmdDispatcher.getPublicCommandSet();
+	}
+	
+	/**
+	 * Returns an array of public command sets. Each set is a categorized subset
+	 * of the command table.
+	 */
+	public Set[] getPublicCommandSubSets() {
+		return cmdDispatcher.getPublicCommandSubSets();
+	}
+	
+	/**
+	 * Returns the localized name of a command subset.
+	 * Indices are defined in CommandDispatcher.
+	 */
+	public String getSubCommandSetName(int index){
+		return cmdDispatcher.getSubCommandSetName(index);
+	}
+	
+
 	final public GeoElement[] processCommand(Command c, boolean labelOutput) throws MyError {
 		return cmdDispatcher.processCommand(c, labelOutput);
 	}
 	
+	/**
+	 * Processes the given casCell, i.e. compute its output depending on 
+	 * its input. Note that this may create an additional twin GeoElement.	 
+	 */
+	final public void processCasCell(GeoCasCell casCell) throws MyError {
+		// TODO: remove
+		System.out.println("*** processCasCell: " + casCell);
+		
+		// check for CircularDefinition
+		if (casCell.isCircularDefinition()) {
+			// set twin geo to undefined
+			casCell.computeOutput(); 
+			casCell.updateCascade();
+			app.showError("CircularDefinition");
+			return;
+		}		
+		
+		AlgoElement algoParent = casCell.getParentAlgorithm();		
+		boolean prevFree = algoParent == null;
+		boolean nowFree = casCell.getGeoElementVariables() == null;				
+		boolean needsRedefinition = false;
+
+		if (prevFree) {
+			if (nowFree) {
+				// free -> free, e.g. m := 7  ->  m := 8
+				cons.addToConstructionList(casCell, true);	
+				casCell.computeOutput(); // create twinGeo if necessary			    	        
+				casCell.setLabelOfTwinGeo();
+				needsRedefinition = false;
+			}
+			else {
+				// free -> dependent, e.g. m := 7  ->  m := c+2	
+				if (casCell.isOutputEmpty() && !casCell.hasChildren()) {
+					// this is a new casCell
+					cons.removeFromConstructionList(casCell);	
+					kernel.DependentCasCell(casCell);
+					needsRedefinition = false;
+				} else {
+					// existing casCell with possible twinGeo
+					needsRedefinition = true;
+				}
+			}			
+		} else {
+			if (nowFree) {
+				// dependent -> free, e.g. m := c+2  ->  m := 7
+				// algorithm will be removed through redefinition 				
+				needsRedefinition = true;
+			}
+			else {
+				// dependent -> dependent, e.g. m := c+2  ->  m := c+d	
+				// we already have an algorithm but need redefinition
+				// in order to move it to the right place in construction list
+				needsRedefinition = true;				
+			}		
+		}
+		
+		if (needsRedefinition) {
+			try {
+				// update construction order and
+				// rebuild construction using XML
+				cons.changeCasCell(casCell);
+			} catch (Exception e) {
+				casCell.setError("RedefinitionFailed");
+				//app.showError(e.getMessage());				
+			}	
+		} else {
+			casCell.updateCascade();
+		}
+	}
+	
 	/**
 	 * for AlgebraView changes in the tree selection and redefine dialog
 	 * @return changed geo
@@ -75,10 +180,27 @@ public class AlgebraProcessor {
 	public GeoElement changeGeoElement(
 			GeoElement geo,
 			String newValue,
-			boolean redefineIndependent) {
+			boolean redefineIndependent, boolean storeUndoInfo) {
 						
 			try {
-				return changeGeoElementNoExceptionHandling(geo, newValue, redefineIndependent, true);
+				return changeGeoElementNoExceptionHandling(geo, newValue, redefineIndependent, storeUndoInfo);
+			} catch (Exception e) {
+				app.showError(e.getMessage());
+				return null;
+			}						
+	}	
+	
+	/**
+	 * for AlgebraView changes in the tree selection and redefine dialog
+	 * @return changed geo
+	 */
+	public GeoElement changeGeoElement(
+			GeoElement geo,
+			ValidExpression newValueVE,
+			boolean redefineIndependent, boolean storeUndoInfo) {
+						
+			try {
+				return changeGeoElementNoExceptionHandling(geo, newValueVE, redefineIndependent, storeUndoInfo);
 			} catch (Exception e) {
 				app.showError(e.getMessage());
 				return null;
@@ -90,24 +212,45 @@ public class AlgebraProcessor {
 	 * @return changed geo
 	 */
 	public GeoElement changeGeoElementNoExceptionHandling(GeoElement geo, String newValue, boolean redefineIndependent, boolean storeUndoInfo) 
+	throws Exception {
+		
+		try {
+			ValidExpression ve = parser.parseGeoGebraExpression(newValue);
+			return changeGeoElementNoExceptionHandling(geo, ve, redefineIndependent, storeUndoInfo);
+		} 
+		catch (Exception e) {
+			e.printStackTrace();
+			throw new Exception(app.getError("InvalidInput") + ":\n" + newValue);
+		} catch (MyError e) {
+			e.printStackTrace();
+			throw new Exception(e.getLocalizedMessage());
+		} catch (Error e) {
+			e.printStackTrace();
+			throw new Exception(app.getError("InvalidInput") + ":\n" + newValue);
+		}
+	}
+	
+	/**
+	 * for AlgebraView changes in the tree selection and redefine dialog
+	 * @return changed geo
+	 */
+	public GeoElement changeGeoElementNoExceptionHandling(GeoElement geo, ValidExpression newValue, boolean redefineIndependent, boolean storeUndoInfo) 
 	throws Exception {
 		String oldLabel, newLabel;
-		ValidExpression ve;
 		GeoElement[] result;
 
 		try {
-			ve = parser.parse(newValue);			
 			oldLabel = geo.getLabel();
-			newLabel = ve.getLabel();
+			newLabel = newValue.getLabel();
 
 			if (newLabel == null) {
 				newLabel = oldLabel;
-				ve.setLabel(newLabel);
+				newValue.setLabel(newLabel);
 			}
 			
 			// make sure that points stay points and vectors stay vectors
-			if (ve instanceof ExpressionNode) {
-				ExpressionNode n = (ExpressionNode) ve;
+			if (newValue instanceof ExpressionNode) {
+				ExpressionNode n = (ExpressionNode) newValue;
 				if (geo.isGeoPoint()) 
 					n.setForcePoint();
 				else if (geo.isGeoVector())
@@ -118,14 +261,14 @@ public class AlgebraProcessor {
 
 			if (newLabel.equals(oldLabel)) {
 				// try to overwrite                
-				result = processValidExpression(ve, redefineIndependent);
+				result = processValidExpression(newValue, redefineIndependent);
 				if (result != null && storeUndoInfo)
 					app.storeUndoInfo();
 				return result[0];
 			} else if (cons.isFreeLabel(newLabel)) {
-				ve.setLabel(oldLabel);
+				newValue.setLabel(oldLabel);
 				// rename to oldLabel to enable overwriting
-				result = processValidExpression(ve, redefineIndependent);
+				result = processValidExpression(newValue, redefineIndependent);
 				result[0].setLabel(newLabel); // now we rename	
 				if (storeUndoInfo)
 					app.storeUndoInfo();
@@ -156,29 +299,62 @@ public class AlgebraProcessor {
 	public GeoElement[] processAlgebraCommand(String cmd, boolean storeUndo) {
 		
 		try {
-			return processAlgebraCommandNoExceptionHandling(cmd, storeUndo);
+			return processAlgebraCommandNoExceptionHandling(cmd, storeUndo,true,false);
 		} catch (Exception e) {
+			e.printStackTrace();
 			app.showError(e.getMessage());
 			return null;
 		}	
 	}
 	
+	
+	// G.Sturr 2010-7-5
+	// normal usage ... default to show error dialog
+	public GeoElement[] processAlgebraCommandNoExceptions(String cmd,
+			boolean storeUndo) {
+
+		try {
+			return processAlgebraCommandNoExceptionHandling(cmd, storeUndo,
+					true, false);
+		} catch (Exception e) {
+			return null;
+		}
+	}
+	
+	public GeoElement[] processAlgebraCommandNoExceptionsOrErrors(String cmd,
+			boolean storeUndo) {
+
+		try {
+			return processAlgebraCommandNoExceptionHandling(cmd, storeUndo,
+					false, false);
+		} catch (Exception e) {
+			return null;
+		} catch (MyError e) {
+			return null;
+		}
+	}
+	
+	// G.Sturr 2010-7-5
+	// added 'allowErrorDialog' flag to handle the case of unquoted text 
+	// entries in the spreadsheet
 	public GeoElement[] processAlgebraCommandNoExceptionHandling(
-			String cmd, 
-			boolean storeUndo) 
+			String cmd, boolean storeUndo, boolean allowErrorDialog, boolean throwMyError) 
 	throws Exception {
 		ValidExpression ve;					
 		
 		try {
-			ve = parser.parse(cmd);
+			ve = parser.parseGeoGebraExpression(cmd);
 		} catch (ParseException e) {
-			e.printStackTrace();
-			throw new Exception(app.getError("InvalidInput") + ":\n" + cmd);
+			//e.printStackTrace();
+			if (allowErrorDialog) {app.showError(app.getError("InvalidInput") + ":\n" + cmd);return null;}
+			throw new MyException(app.getError("InvalidInput") + ":\n" + cmd, MyException.INVALID_INPUT);						
 		} catch (MyError e) {
-			e.printStackTrace();
+			//e.printStackTrace();
+			if (allowErrorDialog) {app.showError(e.getLocalizedMessage());return null;}
 			throw new Exception(e.getLocalizedMessage());
 		} catch (Error e) {
-			e.printStackTrace();
+			//e.printStackTrace();
+			if (allowErrorDialog) {app.showError(app.getError("InvalidInput") + ":\n" + cmd);return null;}
 			throw new Exception(app.getError("InvalidInput") + ":\n" + cmd);
 		}
 
@@ -190,13 +366,23 @@ public class AlgebraProcessor {
 				app.storeUndoInfo();
 		} catch (MyError e) {
 			e.printStackTrace();
-			throw new Exception(e.getLocalizedMessage());
+			//throw new Exception(e.getLocalizedMessage());
+			
+			// show error with nice "Show Online Help" box
+			if(allowErrorDialog) {// G.Sturr 2010-7-5
+				app.showError(e);
+				e.printStackTrace();
+			}
+			else if (throwMyError) throw new MyError(app, e.getLocalizedMessage(), e.getcommandName());
+
+			return null;
 		} catch (CircularDefinitionException e) {
 			Application.debug("CircularDefinition");
 			throw e;
 		} catch (Exception ex) {
+			Application.debug("Exception");
 			ex.printStackTrace();
-			throw new Exception("InvalidInput");
+			throw new Exception(app.getError("Error") + ":\n" + ex.getLocalizedMessage());
 		}
 		return geoElements;
 	}
@@ -206,6 +392,14 @@ public class AlgebraProcessor {
 	 * Returns Double.NaN if something went wrong.
 	 */
 	public double evaluateToDouble(String str) {
+		return evaluateToDouble(str, false);
+	}
+	
+	/**
+	 * Parses given String str and tries to evaluate it to a double.
+	 * Returns Double.NaN if something went wrong.
+	 */
+	public double evaluateToDouble(String str, boolean suppressErrors) {
 		try {
 			ValidExpression ve = parser.parseExpression(str);
 			ExpressionNode en = (ExpressionNode) ve;
@@ -214,15 +408,15 @@ public class AlgebraProcessor {
 			return nv.getDouble();
 		} catch (Exception e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 			return Double.NaN;
 		} catch (MyError e) {
 			e.printStackTrace();
-			app.showError(e);
+			if (!suppressErrors) app.showError(e);
 			return Double.NaN;
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 			return Double.NaN;
 		}
 	}
@@ -237,7 +431,7 @@ public class AlgebraProcessor {
 
 		GeoBoolean bool = null;
 		try {
-			ValidExpression ve = parser.parse(str);		
+			ValidExpression ve = parser.parseGeoGebraExpression(str);		
 			GeoElement [] temp = processValidExpression(ve);
 			bool = (GeoBoolean) temp[0];
 		} catch (CircularDefinitionException e) {
@@ -245,13 +439,13 @@ public class AlgebraProcessor {
 			app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
 			app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
@@ -269,7 +463,7 @@ public class AlgebraProcessor {
 
 		GeoList list = null;
 		try {
-			ValidExpression ve = parser.parse(str);		
+			ValidExpression ve = parser.parseGeoGebraExpression(str);		
 			GeoElement [] temp = processValidExpression(ve);
 			list = (GeoList) temp[0];
 		} catch (CircularDefinitionException e) {
@@ -277,13 +471,13 @@ public class AlgebraProcessor {
 			app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
 			app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
@@ -295,13 +489,13 @@ public class AlgebraProcessor {
 	 * Returns null if something went wrong.
 	 * Michael Borcherds 2008-04-04
 	 */
-	public GeoFunction evaluateToFunction(String str) {
+	public GeoFunction evaluateToFunction(String str, boolean suppressErrors) {
 		boolean oldMacroMode = cons.isSuppressLabelsActive();
 		cons.setSuppressLabelCreation(true);
 
 		GeoFunction func = null;
 		try {
-			ValidExpression ve = parser.parse(str);		
+			ValidExpression ve = parser.parseGeoGebraExpression(str);		
 			GeoElement [] temp = processValidExpression(ve);
 			
 			if (temp[0].isGeoFunctionable()) {
@@ -309,20 +503,20 @@ public class AlgebraProcessor {
 				func = f.getGeoFunction();
 			}						
 			else 
-				app.showError("InvalidInput");
+				if (!suppressErrors) app.showError("InvalidInput", str);
 			
 		} catch (CircularDefinitionException e) {
 			Application.debug("CircularDefinition");
-			app.showError("CircularDefinition");
+			if (!suppressErrors) app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
-			app.showError(e);
+			if (!suppressErrors) app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
@@ -334,27 +528,27 @@ public class AlgebraProcessor {
 	 * Returns null if something went wrong.
 	 * Michael Borcherds 2008-08-13
 	 */
-	public NumberValue evaluateToNumeric(String str) {
+	public NumberValue evaluateToNumeric(String str, boolean suppressErrors) {
 		boolean oldMacroMode = cons.isSuppressLabelsActive();
 		cons.setSuppressLabelCreation(true);
 
 		NumberValue num = null;
 		try {
-			ValidExpression ve = parser.parse(str);		
+			ValidExpression ve = parser.parseGeoGebraExpression(str);		
 			GeoElement [] temp = processValidExpression(ve);
 			num = (NumberValue) temp[0];
 		} catch (CircularDefinitionException e) {
 			Application.debug("CircularDefinition");
-			app.showError("CircularDefinition");
+			if (!suppressErrors) app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
-			app.showError(e);
+			if (!suppressErrors) app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			if (!suppressErrors) app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
@@ -365,33 +559,33 @@ public class AlgebraProcessor {
 	 * Parses given String str and tries to evaluate it to a GeoPoint.
 	 * Returns null if something went wrong.
 	 */
-	public GeoPoint evaluateToPoint(String str) {
+	public GeoPointND evaluateToPoint(String str) {
 		boolean oldMacroMode = cons.isSuppressLabelsActive();
 		cons.setSuppressLabelCreation(true);
 
-		GeoPoint p = null;
+		GeoPointND p = null;
 		GeoElement [] temp = null;;
 		try {
-			ValidExpression ve = parser.parse(str);
+			ValidExpression ve = parser.parseGeoGebraExpression(str);
 			if (ve instanceof ExpressionNode) {
 				ExpressionNode en = (ExpressionNode) ve;
 				en.setForcePoint();	
 			}
 			 
 			 temp = processValidExpression(ve);
-			 p = (GeoPoint) temp[0];
+			 p = (GeoPointND) temp[0];
 		} catch (CircularDefinitionException e) {
 			Application.debug("CircularDefinition");
 			app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
 			app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
@@ -409,7 +603,7 @@ public class AlgebraProcessor {
 		GeoText text = null;
 		GeoElement [] temp = null;;
 		try {
-			ValidExpression ve = parser.parse(str);			
+			ValidExpression ve = parser.parseGeoGebraExpression(str);			
 			temp = processValidExpression(ve);
 			text = (GeoText) temp[0];
 		} catch (CircularDefinitionException e) {
@@ -417,19 +611,53 @@ public class AlgebraProcessor {
 			app.showError("CircularDefinition");
 		} catch (Exception e) {		
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} catch (MyError e) {
 			e.printStackTrace();
 			app.showError(e);
 		} catch (Error e) {
 			e.printStackTrace();
-			app.showError("InvalidInput");
+			app.showError("InvalidInput", str);
 		} 
 		
 		cons.setSuppressLabelCreation(oldMacroMode);
 		return text;
 	}
 
+	/**
+	 * Parses given String str and tries to evaluate it to a GeoImplicitPoly object.
+	 * Returns null if something went wrong.
+	 * @param str 
+	 * @boolean showErrors if false, only stacktraces are printed
+	 * @return implicit polygon or null
+	 */
+	public GeoElement evaluateToGeoElement(String str,boolean showErrors) {
+		boolean oldMacroMode = cons.isSuppressLabelsActive();
+		cons.setSuppressLabelCreation(true);
+
+		GeoElement geo = null;
+		try {
+			ValidExpression ve = parser.parseGeoGebraExpression(str);		
+			GeoElement [] temp = processValidExpression(ve);
+			geo = temp[0];			
+		} catch (CircularDefinitionException e) {
+			Application.debug("CircularDefinition");
+			app.showError("CircularDefinition");
+		} catch (Exception e) {		
+			e.printStackTrace();
+			if(showErrors)app.showError("InvalidInput", str);			
+		} catch (MyError e) {
+			e.printStackTrace();
+			if(showErrors)app.showError(e);
+		} catch (Error e) {
+			e.printStackTrace();
+			if(showErrors)app.showError("InvalidInput", str);			 
+		} 
+		
+		cons.setSuppressLabelCreation(oldMacroMode);
+		return geo;
+	}
+	
 	/**
 	 * Checks if label is valid.	 
 	 */
@@ -490,37 +718,12 @@ public class AlgebraProcessor {
 		// we have to make sure that the macro mode is
 		// set back at the end
 		try {
-			if (ve instanceof ExpressionNode) {
-				ret = processExpressionNode((ExpressionNode) ve);
-			}
-	
-			// Command		
-			else if (ve instanceof Command) {
-				ret = cmdDispatcher.processCommand((Command) ve, true);
-			}
-	
-			// Equation in x,y (linear or quadratic are valid): line or conic
-			else if (ve instanceof Equation) {
-				ret = processEquation((Equation) ve);
-			}
-	
-			// explicit Function in x
-			else if (ve instanceof Function) {
-				ret = processFunction(null, (Function) ve);	
-			}						
-	
-			// Parametric Line        
-			else if (ve instanceof Parametric) {
-				ret = processParametric((Parametric) ve);
-			}
-	
-//			// Assignment: variable
-//			else if (ve instanceof Assignment) {
-//				ret = processAssignment((Assignment) ve);
-//			} 
+			ret = doProcessValidExpression(ve);
 			
-			else
-				throw new MyError(app, "Unhandled ValidExpression : " + ve);
+			if (ret == null) { // eg (1,2,3) running in 2D
+				Application.debug("Unhandled ValidExpression : " + ve);
+				throw new MyError(app, app.getError("InvalidInput") + ":\n" + ve);
+			}
 		}
 		finally {
 			cons.setSuppressLabelCreation(oldMacroMode);
@@ -584,8 +787,52 @@ public class AlgebraProcessor {
 			
 		return ret;
 	}
+	
+	public GeoElement [] doProcessValidExpression(ValidExpression ve) throws MyError, Exception {
+		GeoElement [] ret = null;	
+		
+			if (ve instanceof ExpressionNode) {
+				ret = processExpressionNode((ExpressionNode) ve);
+			}
+	
+			// Command		
+			else if (ve instanceof Command) {
+				ret = cmdDispatcher.processCommand((Command) ve, true);
+			}
+			
+			
+			
+			// Equation in x,y (linear or quadratic are valid): line or conic
+			else if (ve instanceof Equation) {
+				ret = processEquation((Equation) ve);
+			}
+	
+			// explicit Function in one variable
+			else if (ve instanceof Function) {
+				ret = processFunction(null, (Function) ve);
+			}	
+			
+			// explicit Function in multiple variables
+			else if (ve instanceof FunctionNVar) {
+				ret = processFunctionNVar(null, (FunctionNVar) ve);
+			}	
+	
+			// Parametric Line        
+			else if (ve instanceof Parametric) {
+				ret = processParametric((Parametric) ve);
+			}
+	
+//			// Assignment: variable
+//			else if (ve instanceof Assignment) {
+//				ret = processAssignment((Assignment) ve);
+//			} 
+			
 
-	private GeoElement[] processFunction(ExpressionNode funNode, Function fun) {		
+		return ret;
+	}
+	
+
+	protected GeoElement[] processFunction(ExpressionNode funNode, Function fun) {		
 		fun.initFunction();		
 		
 		String label = fun.getLabel();
@@ -595,6 +842,90 @@ public class AlgebraProcessor {
 		GeoElement[] vars = fun.getGeoElementVariables();				
 		boolean isIndependent = (vars == null || vars.length == 0);
 		
+		// check for interval
+		
+		ExpressionNode en = fun.getExpression();
+		if (en.operation == en.AND) {
+			ExpressionValue left = en.left;
+			ExpressionValue right = en.right;
+			
+			if (left.isExpressionNode() && right.isExpressionNode()) {
+				ExpressionNode enLeft = (ExpressionNode)left;
+				ExpressionNode enRight = (ExpressionNode)right;
+				
+				int opLeft = enLeft.operation;
+				int opRight = enRight.operation;
+				
+				ExpressionValue leftLeft = enLeft.left;
+				ExpressionValue leftRight = enLeft.right;
+				ExpressionValue rightLeft = enRight.left;
+				ExpressionValue rightRight = enRight.right;
+				
+				// directions of inequalities, need one + and one - for an interval
+				int leftDir = 0;
+				int rightDir = 0;
+				
+	
+				if ((opLeft == en.LESS || opLeft == en.LESS_EQUAL)) {
+					if (leftLeft instanceof FunctionVariable && leftRight.isNumberValue()) leftDir = -1;
+					else if (leftRight instanceof FunctionVariable && leftLeft.isNumberValue()) leftDir = +1;
+					
+				} else
+				if ((opLeft == en.GREATER || opLeft == en.GREATER_EQUAL)) {
+					if (leftLeft instanceof FunctionVariable && leftRight.isNumberValue()) leftDir = +1;
+					else if (leftRight instanceof FunctionVariable && leftLeft.isNumberValue()) leftDir = -1;
+					
+				}
+				
+				if ((opRight == en.LESS || opRight == en.LESS_EQUAL)) {
+					if (rightLeft instanceof FunctionVariable && rightRight.isNumberValue()) rightDir = -1;
+					else if (rightRight instanceof FunctionVariable && rightLeft.isNumberValue()) rightDir = +1;
+					
+				} else
+				if ((opRight == en.GREATER || opRight == en.GREATER_EQUAL)) {
+					if (rightLeft instanceof FunctionVariable && rightRight.isNumberValue()) rightDir = +1;
+					else if (rightRight instanceof FunctionVariable && rightLeft.isNumberValue()) rightDir = -1;
+					
+				}
+				
+				//Application.debug(leftDir+" "+rightDir);
+				//Application.debug(leftLeft.getClass()+" "+leftRight.getClass());
+				//Application.debug(rightLeft.getClass()+" "+rightRight.getClass());
+				
+				// opposite directions -> OK
+				if (leftDir * rightDir < 0) {
+					if (isIndependent) {
+						f = kernel.Interval(label, fun);			
+					} else {			
+						f = kernel.DependentInterval(label, fun);
+					}
+					ret[0] = f;		
+					return ret;
+
+				}
+				
+				
+				//Application.debug(enLeft.operation+"");
+				//Application.debug(enLeft.left.getClass()+"");
+				//Application.debug(enLeft.right.getClass()+"");
+				
+
+			}
+			//Application.debug(left.getClass()+"");
+			//Application.debug(right.getClass()+"");
+			//Application.debug("");
+		} else if (en.operation == en.FUNCTION) {
+			ExpressionValue left = en.left;
+			ExpressionValue right = en.right;
+			if (left.isLeaf() && left.isGeoElement() &&
+				right.isLeaf() && right.isNumberValue() &&
+				!isIndependent) {
+				f = (GeoFunction) kernel.DependentGeoCopy(label, (GeoElement)left);
+				ret[0] = f;		
+				return ret;
+			}
+		}
+
 		if (isIndependent) {
 			f = kernel.Function(label, fun);			
 		} else {			
@@ -603,26 +934,59 @@ public class AlgebraProcessor {
 		ret[0] = f;		
 		return ret;
 	}
+	
+	protected GeoElement[] processFunctionNVar(ExpressionNode funNode, FunctionNVar fun) {		
+		fun.initFunction();		
+		
+		String label = fun.getLabel();
+		GeoElement[] ret = new GeoElement[1];
 
-	private GeoElement[] processEquation(Equation equ) throws MyError {		
+		GeoElement[] vars = fun.getGeoElementVariables();				
+		boolean isIndependent = (vars == null || vars.length == 0);		
+		
+		if (isIndependent) {				
+			ret[0] = kernel.FunctionNVar(label, fun );			
+		} else {
+			ret[0] = kernel.DependentFunctionNVar(label, fun);
+		}
+		return ret;
+	}
+
+	public GeoElement[] processEquation(Equation equ) throws MyError {		
 //		Application.debug("EQUATION: " + equ);        
 //		Application.debug("NORMALFORM POLYNOMIAL: " + equ.getNormalForm());        		
 		
+		
 		try {
 			equ.initEquation();	
+			//Application.debug("EQUATION: " + equ.getNormalForm());    	
+			// check no terms in z
+			checkNoTermsInZ(equ);
+			
+			if (equ.isFunctionDependent()){
+				return processImplicitPoly(equ);
+			}
 
-			// consider algebraic degree of equation        
+			// consider algebraic degree of equation  
+			 // check not equation of eg plane
 			switch (equ.degree()) {
 				// linear equation -> LINE   
 				case 1 :
-					return processLine(equ);
+					return processLine(equ, false);
 	
 				// quadratic equation -> CONIC                                  
 				case 2 :
 					return processConic(equ);
 	
 				default :
-					throw new MyError(app, "InvalidEquation");
+					//test for "y= <rhs>" here as well
+					if (equ.getLHS().toString().trim().equals("y")){
+						Function fun = new Function(equ.getRHS());
+						// try to use label of equation							
+						fun.setLabel(equ.getLabel());
+						return processFunction(null, fun);
+					}
+					return processImplicitPoly(equ);
 			}
 		} 
 		catch (MyError eqnError) {
@@ -652,9 +1016,18 @@ public class AlgebraProcessor {
 				throw new MyError(app, errors);
 			}
         }        
+		
+		
+	}
+	
+	
+	
+	protected void checkNoTermsInZ(Equation equ){
+		if (!equ.getNormalForm().isFreeOf('z')) 
+			throw new MyError(app, "InvalidEquation");
 	}
 
-	private GeoElement[] processLine(Equation equ) {
+	protected GeoElement[] processLine(Equation equ, boolean inequality) {
 		double a = 0, b = 0, c = 0;
 		GeoLine line;
 		GeoElement[] ret = new GeoElement[1];
@@ -668,9 +1041,9 @@ public class AlgebraProcessor {
 			a = lhs.getCoeffValue("x");
 			b = lhs.getCoeffValue("y");
 			c = lhs.getCoeffValue("");
-			line = kernel.Line(label, a, b, c);
+			line =  kernel.Line(label, a, b, c);
 		} else
-			line = kernel.DependentLine(label, equ);
+			line =  kernel.DependentLine(label, equ);
 
 		if (isExplicit) {
 			line.setToExplicit();
@@ -680,12 +1053,13 @@ public class AlgebraProcessor {
 		return ret;
 	}
 
-	private GeoElement[] processConic(Equation equ) {
+	protected GeoElement[] processConic(Equation equ) {
 		double a = 0, b = 0, c = 0, d = 0, e = 0, f = 0;
 		GeoElement[] ret = new GeoElement[1];
 		GeoConic conic;
 		String label = equ.getLabel();
 		Polynomial lhs = equ.getNormalForm();
+		
 		boolean isExplicit = equ.isExplicit("y");
 		boolean isSpecific =
 			!isExplicit && (equ.isExplicit("yy") || equ.isExplicit("xx"));
@@ -712,6 +1086,29 @@ public class AlgebraProcessor {
 		return ret;
 	}
 
+	protected GeoElement[] processImplicitPoly(Equation equ){
+		GeoElement[] ret = new GeoElement[1];
+		String label = equ.getLabel();
+		Polynomial lhs = equ.getNormalForm();
+		boolean isIndependent = !equ.isFunctionDependent()&&lhs.isConstant();
+		GeoImplicitPoly poly;
+		GeoElement geo=null;
+		if (isIndependent){
+			poly=kernel.ImplicitPoly(label, lhs);
+			poly.setUserInput(equ);
+			geo=poly;
+		}else{
+			geo=kernel.DependentImplicitPoly(label, equ); //might also return Line or Conic
+			if (geo instanceof GeoUserInputElement){
+				((GeoUserInputElement)geo).setUserInput(equ);
+			}
+		}
+		ret[0]=geo;
+//		Application.debug("User Input: "+equ);
+		ret[0].updateRepaint();
+		return ret;
+	}
+
 	private GeoElement[] processParametric(Parametric par)
 		throws CircularDefinitionException {
 		
@@ -764,27 +1161,50 @@ public class AlgebraProcessor {
 		GeoElement[] ret = { line };
 		return ret;
 	}
+	
+	
+
 
 	protected GeoElement[] processExpressionNode(ExpressionNode n) throws MyError {					
-		// command is leaf: process command
-		if (n.isLeaf() && n.getLeft() instanceof Command) {
-			Command c = (Command) n.getLeft();
-			c.setLabels(n.getLabels());
-			return cmdDispatcher.processCommand(c, true);
-		}			
+		// command is leaf: process command		
+		if (n.isLeaf()) {			
+			 ExpressionValue leaf =  n.getLeft();
+			 if (leaf instanceof Command) {			
+				Command c = (Command) leaf;
+				c.setLabels(n.getLabels());
+				return cmdDispatcher.processCommand(c, true);
+			 }
+			 else if (leaf instanceof Equation) {
+				 Equation eqn = (Equation) leaf;
+				 eqn.setLabels(n.getLabels());
+				 return processEquation(eqn);
+			 }
+			 else if (leaf instanceof Function) {				 
+				Function fun = (Function) leaf;
+				fun.setLabels(n.getLabels());
+				return processFunction(n, fun);			
+			} 
+			 else if (leaf instanceof FunctionNVar) {
+				FunctionNVar fun = (FunctionNVar) leaf;
+				fun.setLabels(n.getLabels());
+				return processFunctionNVar(n, fun);			
+			} 
+			 
+			 
+		}											
 		
 		// ELSE:  resolve variables and evaluate expressionnode		
 		n.resolveVariables();			
-		eval = n.evaluate(); //ggb3D : used by AlgebraProcessor3D in extended processExpressionNode		
+		eval = n.evaluate(); 
 		boolean dollarLabelFound = false;		
 		
+		ExpressionNode myNode = n;
+		if (myNode.isLeaf()) myNode = myNode.getLeftTree();
 		// leaf (no new label specified): just return the existing GeoElement
-		if (eval.isGeoElement() &&  n.getLabel() == null) 
+		if (eval.isGeoElement() &&  n.getLabel() == null && !(n.operation == ExpressionNode.ELEMENT_OF)) 
 		{
 			// take care of spreadsheet $ names: don't loose the wrapper ExpressionNode here
 			// check if we have a Variable 
-			ExpressionNode myNode = n;
-			if (myNode.isLeaf()) myNode = myNode.getLeftTree();
 			switch (myNode.getOperation()) {
 				case ExpressionNode.$VAR_COL:
 				case ExpressionNode.$VAR_ROW:
@@ -796,7 +1216,7 @@ public class AlgebraProcessor {
 					
 				default:
 					// return the GeoElement
-					GeoElement[] ret = {(GeoElement) eval };
+					GeoElement[] ret = {(GeoElement) eval };					
 					return ret;
 			}			
 		}		
@@ -807,18 +1227,36 @@ public class AlgebraProcessor {
 			return processNumber(n, eval);
 		else if (eval.isVectorValue())
 			return processPointVector(n, eval);	
+		else if (eval.isVector3DValue())
+			return processPointVector3D(n, eval);	
 		else if (eval.isTextValue())
-			return processText(n, eval);		
-		else if (eval instanceof Function) {
-			return processFunction(n, (Function) eval);			
-		} 		
+			return processText(n, eval);				
 		else if (eval instanceof MyList) {
-			return processList(n, (MyList) eval);
+			return processList(n, (MyList) eval);		
+		} else if (eval instanceof Function) {			
+			return processFunction(n, (Function) eval);			
 		} 
+		else if (eval instanceof FunctionNVar) {
+			
+			return processFunctionNVar(n, (FunctionNVar) eval);			
+		} 
+		//we have to process list in case list=matrix1(1), but not when list=list2 
+		else if (eval instanceof GeoList  && myNode.hasOperations()) {
+			Application.debug("should work");
+			return processList(n, ((GeoList) eval).getMyList());
+		} else if (eval.isGeoElement()) {	
+
+			// e.g. B1 = A1 where A1 is a GeoElement and B1 does not exist yet
+			// create a copy of A1
+				if (n.getLabel() != null || dollarLabelFound) {
+					return processGeoCopy(n.getLabel(), n);	
+				}									
+			} 	
+
 		
 		// REMOVED due to issue 131: http://code.google.com/p/geogebra/issues/detail?id=131
 //		// expressions like 2 a (where a:x + y = 1)
-//		//TODO A1=b doesn't work for these objects
+//		// A1=b doesn't work for these objects
 //		else if (eval instanceof GeoLine) {
 //			if (((GeoLine)eval).getParentAlgorithm() instanceof AlgoDependentLine) {
 //				GeoElement[] ret = {(GeoElement) eval };
@@ -826,14 +1264,7 @@ public class AlgebraProcessor {
 //			}
 // 
 //		}
-		
-		// e.g. B1 = A1 where A1 is a GeoElement and B1 does not exist yet
-		// create a copy of A1
-		else if (eval.isGeoElement()) {
-			if (n.getLabel() != null || dollarLabelFound) {
-				return processGeoCopy(n.getLabel(), n);	
-			}									
-		}		
+		 
 		
 		// if we get here, nothing worked
 		Application.debug(
@@ -886,7 +1317,7 @@ public class AlgebraProcessor {
 			
 			int size = evalList.size();
 			for (int i=0; i < size; i++) {
-				ExpressionNode en = evalList.getListElement(i);
+				ExpressionNode en = (ExpressionNode) evalList.getListElement(i);
 				// we only take one resulting object	
 				GeoElement [] results = processExpressionNode(en);						
 				GeoElement geo = results[0];										
@@ -957,26 +1388,20 @@ public class AlgebraProcessor {
 		GeoVec3D[] ret = new GeoVec3D[1];
 		boolean isIndependent = n.isConstant();
 
+		// make point if complex parts are present, e.g. 3 + i
+		if (complex) {
+			n.setForcePoint();
+		}
 		// make vector, if label begins with lowercase character
-		if (label != null) {
+		else if (label != null) {
 			if (!(n.isForcedPoint() || n.isForcedVector())) { // may be set by MyXMLHandler
-				if (!complex && Character.isLowerCase(label.charAt(0)))
+				if (Character.isLowerCase(label.charAt(0)))
 					n.setForceVector();
 				else
 					n.setForcePoint();
 			}
 		}
 		boolean isVector = n.isVectorValue();
-		
-		// check for free "complex" points like 2 + i that only depend on i
-		if (complex && !isIndependent) {
-			GeoElement [] vars = n.getGeoElementVariables();
-			if (vars.length == 1 && "i".equals(vars[0].getLabel())) {
-				isIndependent = true;			
-				complex = true;
-			}
-	
-		}
 
 		if (isIndependent) {
 			// get coords
@@ -1001,7 +1426,19 @@ public class AlgebraProcessor {
 		}
 		return ret;
 	}
-	
+		
+	/** empty method in 2D : see AlgebraProcessor3D to see implementation in 3D
+	 * @param n
+	 * @param evaluate
+	 * @return null
+	 */
+	protected GeoElement[] processPointVector3D(
+			ExpressionNode n,
+			ExpressionValue evaluate) {
+
+		return null;
+	}
+		
 	/** 
 	 * Creates a dependent copy of origGeo with label
 	 */
@@ -1011,6 +1448,11 @@ public class AlgebraProcessor {
 		return ret;
 	}
 
+	/** Enables CAS specific behaviour */
+	public void enableCAS() {
+		cmdDispatcher.initCASCommands();
+	}
+
 //	/**
 //	 * Processes assignments, i.e. input of the form leftVar = geoRight where geoRight is an existing GeoElement.
 //	 */
