@@ -12,8 +12,8 @@ the Free Software Foundation.
 
 package geogebra.kernel.statistics;
 
-import geogebra.kernel.GeoList;
 import geogebra.kernel.Construction;
+import geogebra.kernel.GeoList;
 
 /**
  * Sxy of a list
@@ -29,7 +29,7 @@ public class AlgoDoubleListSXY extends AlgoStats2D {
         super(cons,label,geoListx,geoListy,AlgoStats2D.STATS_SXY);
     }
 
-    protected String getClassName() {
+    public String getClassName() {
         return "AlgoDoubleListSXY";
     }
 }
