@@ -11,7 +11,9 @@ package geogebra.plugin;
  
  */
 import geogebra.GeoGebra;
+import geogebra.cas.GeoGebraCAS;
 import geogebra.euclidian.EuclidianView;
+import geogebra.io.MyImageIO;
 import geogebra.kernel.Construction;
 import geogebra.kernel.GeoBoolean;
 import geogebra.kernel.GeoElement;
@@ -22,7 +24,6 @@ import geogebra.kernel.GeoVector;
 import geogebra.kernel.Kernel;
 import geogebra.kernel.PointProperties;
 import geogebra.kernel.Traceable;
-import geogebra.kernel.arithmetic.MyBoolean;
 import geogebra.kernel.arithmetic.NumberValue;
 import geogebra.kernel.commands.AlgebraProcessor;
 import geogebra.main.Application;
@@ -31,15 +32,21 @@ import java.awt.Color;
 import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.io.ByteArrayOutputStream;
+import java.io.File;
 import java.io.IOException;
 import java.net.URL;
+import java.security.AccessController;
 import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
+import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.Locale;
 import java.util.TreeSet;
 
 import javax.imageio.ImageIO;
+import javax.imageio.ImageWriter;
+import javax.imageio.stream.ImageOutputStream;
 
 
 /** 
@@ -135,10 +142,11 @@ public class GgbAPI {
     
     /** Making MathPiper available for Plugins
      *  (Silent version, without debug to console)
-     */
+     *  @deprecated since GeoGebra 4.0
+     *
     public String evaluateMathPiper(String cmdString){
     	return kernel.evaluateMathPiper(cmdString);
-    }//evaluateMathPiper(String)
+    }//evaluateMathPiper(String)*/
     	
     /// --- 29.05.08 Ulven: --- ///
     
@@ -168,6 +176,20 @@ public class GgbAPI {
 		return app.getXML();
 	}
 	
+	/**
+	 * Returns current construction in Base64 format. May be used for saving.
+	 */
+	public synchronized String getBase64(boolean includeThumbnail) {
+		ByteArrayOutputStream baos = new ByteArrayOutputStream();
+		try {
+			app.getXMLio().writeGeoGebraFile(baos, includeThumbnail);
+			return geogebra.util.Base64.encode(baos.toByteArray(), 0);
+		} catch (IOException e) {
+			e.printStackTrace();
+			return null;
+		}
+	}
+	
 	/**
 	 * Returns the GeoGebra XML string for the given GeoElement object, 
 	 * i.e. only the <element> tag is returned. 
@@ -208,12 +230,27 @@ public class GgbAPI {
 		app.setXML(xml, true);
 	}
 	
+	/**
+	 * Opens construction given in XML format. May be used for loading constructions.
+	 */
+	public synchronized void setBase64(String base64) {
+		byte[] zipFile;
+		try {
+			zipFile = geogebra.util.Base64.decode(base64);
+		} catch (IOException e) {
+			
+			e.printStackTrace();
+			return;
+		}
+		app.loadXML(zipFile);
+	}
+	
 	/**
 	 * Evaluates the given XML string and changes the current construction. 
 	 * Note: the construction is NOT cleared before evaluating the XML string. 	 
 	 */
 	public synchronized void evalXML(String xmlString) {		
-		StringBuffer sb = new StringBuffer();
+		StringBuilder sb = new StringBuilder();
 		
 		sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
 		sb.append("<geogebra format=\"" + GeoGebra.XML_FILE_FORMAT + "\">\n");
@@ -253,19 +290,76 @@ public class GgbAPI {
 	/**
 	 * Evaluates the given string as if it was entered into MathPiper's 
 	 * input text field. 	 
-	 */
+	 * @deprecated since GeoGebra 4.0, use evalGeoGebraCAS() instead
+	 *
 	public synchronized String evalMathPiper(String cmdString) {
 		
 		String ret = kernel.evaluateMathPiper(cmdString);
 		
 		// useful for debugging JavaScript
 		// do not remove!
-		Application.debug("evalMathPiper\n input:"+cmdString+"\n\t"+"output: "+ret);
+		Application.debug("evalMathPiper\n input:"+cmdString+"\n"+"output: "+ret);
 		
 		return ret;
 
-	}
+	}*/
 
+	/**
+	 * Evaluates the given string as if it was entered into Maxima's
+	 * input text field. 
+	 * @author Ulven
+	 * @version 2010-03-10	
+	 * @deprecated since GeoGebra 4.0, use evalGeoGebraCAS() instead 
+	 *
+	public synchronized String evalMaxima(String cmdString) {
+		
+		String ret = kernel.evaluateMaxima(cmdString);
+		
+		// useful for debugging JavaScript
+		// do not remove!
+		Application.debug("evalMaxima\n input:"+cmdString+"\n"+"output: "+ret);
+		
+		return ret;
+
+	}//evalMaxima(String cmdString)*/
+	
+	/* Not needed, see next command:
+	public synchronized geogebra.cas.GeoGebraCAS getCurrentCAS(){
+		return 	(geogebra.cas.GeoGebraCAS) kernel.getGeoGebraCAS();
+	}//getCurrentCas()
+	*/
+	
+	/**
+	 * Evaluates the given string as if it was entered into GeoGebra CAS's
+	 * input text field.  
+	 * @return evaluation result in GeoGebraCAS syntax
+	 */
+	public synchronized String evalGeoGebraCAS(String cmdString){
+		return evalGeoGebraCAS(cmdString, false);		
+	}
+		
+	/**
+	 * Evaluates the given string as if it was entered into GeoGebra CAS's
+	 * input text field. 
+	 * @param debugOutput states whether debugging information should be printed to the console
+	 * @return evaluation result in GeoGebraCAS syntax
+	 */
+	public synchronized String evalGeoGebraCAS(String cmdString, boolean debugOutput) {
+		String ret="";
+		GeoGebraCAS	ggbcas=(GeoGebraCAS)kernel.getGeoGebraCAS();
+		try{
+			ret= ggbcas.evaluateGeoGebraCAS(cmdString);
+		}catch(Throwable t){
+			Application.debug(t.toString());
+		}//try-catch
+		
+		// useful for debugging JavaScript
+		if (debugOutput)
+			Application.debug("evalGeoGebraCAS\n input:"+cmdString+"\n"+"output: "+ret);
+		return ret;
+	}//evalGeoGebraCAS(String)
+	
+	
 	/**
 	 * prints a string to the Java Console
 	 */
@@ -291,14 +385,12 @@ public class GgbAPI {
 		//rewrite in this context
 	}
 	
-	
 	/**
 	 * Clears the construction and resets all views.
 	 */
 	public synchronized void fileNew() {
 		app.fileNew();
 	}
-
 	
 	/**
 	 * Refreshs all views. Note: clears traces in
@@ -577,7 +669,7 @@ public class GgbAPI {
 			ByteArrayOutputStream baos = new ByteArrayOutputStream();
 			ImageIO.write(img, format.toLowerCase(Locale.US), baos);
 			
-			MessageDigest md5 = Application.getMessageDigestMD5();
+			MessageDigest md5 = getMessageDigestMD5();
 			byte[] bytesOut = baos.toByteArray();
 			md5.update(bytesOut);
 			byte[] md5hash = new byte[32];
@@ -590,6 +682,17 @@ public class GgbAPI {
 		}
 		
 	}
+	
+	private static MessageDigest messageDigestMD5 = null;
+
+	public static MessageDigest getMessageDigestMD5()
+			throws NoSuchAlgorithmException {
+		if (messageDigestMD5 == null)
+			messageDigestMD5 = MessageDigest.getInstance("MD5");
+
+		return messageDigestMD5;
+	}
+	
 
 	public synchronized int getLineStyle(String objName) {
 		GeoElement geo = kernel.lookupLabel(objName);
@@ -667,16 +770,27 @@ public class GgbAPI {
 			return geo.isDefined();
 	}	
 	
+	/**
+	 * Returns true if the object with the given name is independent.
+	 */
+	public synchronized boolean isIndependent(String objName) {			
+		GeoElement geo = kernel.lookupLabel(objName);
+		if (geo == null) 
+			return false;
+		else
+			return geo.isIndependent();
+	}	
+	
 	/**
 	 * Returns the value of the object with the given name as a string.
 	 */
 	public synchronized String getValueString(String objName) {
 		GeoElement geo = kernel.lookupLabel(objName);
-		if (geo == null) return "";		
-
+		if (geo == null) return "";	
+		
 		if (geo.isGeoText())
 			return ((GeoText)geo).getTextString();
-
+		
 		return geo.getAlgebraDescription();
 	}
 	
@@ -773,7 +887,7 @@ public class GgbAPI {
 	 */
 	public synchronized void setValue(String objName, double x) {
 		GeoElement geo = kernel.lookupLabel(objName);
-		if (geo == null) return;
+		if (geo == null || !geo.isIndependent()) return;
 		
 		if (geo.isGeoNumeric()) {
 			((GeoNumeric) geo).setValue(x);
@@ -818,9 +932,8 @@ public class GgbAPI {
 	 * Shows or hides the x- and y-axis of the coordinate system in the graphics window.
 	 */
 	public synchronized void setAxesVisible(boolean xVisible, boolean yVisible) {		
-		app.getEuclidianView().showAxes(xVisible, yVisible);
-		//app.getEuclidianView().updateBackground();
-		//app.refreshViews();
+		app.getEuclidianView().setShowAxis(EuclidianView.AXIS_X, xVisible, false);
+		app.getEuclidianView().setShowAxis(EuclidianView.AXIS_Y, yVisible, false);
 		kernel.notifyRepaint();
 	}	
 	
@@ -837,7 +950,8 @@ public class GgbAPI {
 	 * Shows or hides the coordinate grid in the graphics window.
 	 */
 	public synchronized void setGridVisible(boolean flag) {		
-		app.getEuclidianView().showGrid(flag);
+		app.getSettings().getEuclidian(1).showGrid(flag);
+		app.getSettings().getEuclidian(2).showGrid(flag);
 	}
 	
 	/*
@@ -899,7 +1013,7 @@ public class GgbAPI {
 			objNames[i] = geo.getLabel();
 		}
 		return objNames;
-	}
+	}	
 	
 	/**
 	 * Returns the number of objects in the construction.
@@ -921,6 +1035,77 @@ public class GgbAPI {
 		}
 	}
 	
+	
+	/*
+	 * saves a PNG file
+	 * signed applets only
+	 */
+	public synchronized boolean writePNGtoFile(String filename, final double exportScale, final boolean transparent, final double DPI) {
+		if (!app.hasFullPermissions()) return false;
+		final File file = new File(filename);
+
+		if (file == null) return false;
+
+		return (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
+			public Boolean run() {
+
+				try {			
+					// draw graphics view into image
+					BufferedImage img =
+						app.getEuclidianView().getExportImage(exportScale, transparent); 
+					
+					// write image to file
+					MyImageIO.write(img, "png", (float)DPI,  file);	
+					
+					return true;
+				} catch (Exception ex) {
+					Application.debug(ex.toString());
+					return false;
+				} catch (Error ex) {
+					Application.debug(ex.toString());
+					return false;
+				} 
+
+			}
+		});
+
+		
+		
+		
+	}
+	
+	/*
+	 * returns a String (base-64 encoded PNG file of the Graphics View)
+	 */
+	public synchronized String getPNGBase64(double exportScale, boolean transparent, double DPI) {
+		BufferedImage img =
+			app.getEuclidianView().getExportImage(exportScale, transparent); 
+
+		
+	    try {
+		    Iterator it = ImageIO.getImageWritersByFormatName("png");
+		    ImageWriter writer = (ImageWriter) it.next();
+		    ByteArrayOutputStream baos = new ByteArrayOutputStream(); 
+
+		    ImageOutputStream ios = ImageIO.createImageOutputStream(baos);
+		    
+		    writer.setOutput(ios);
+		
+		    MyImageIO.writeImage(writer, img, DPI);
+		    
+		    byte[] image = baos.toByteArray();
+			String ret = geogebra.util.Base64.encode(baos.toByteArray(), 0);
+	    
+			baos.close();
+			
+			return ret;
+		} catch (IOException e) {
+			e.printStackTrace();
+			return null;
+		}
+	}
+	
+	
 	/**
 	 * Returns the type of the object with the given name as a string (e.g. point, line, circle, ...)
 	 */
@@ -936,8 +1121,64 @@ public class GgbAPI {
 		app.setMode(mode);
 	}	
 	
-	
+	public synchronized void registerAddListener(String JSFunctionName) {
+		app.getScriptManager().registerAddListener(JSFunctionName);
+	}
 
+	public synchronized void unregisterAddListener(String JSFunctionName) {
+		app.getScriptManager().unregisterAddListener(JSFunctionName);
+	}
+	
+	public synchronized void registerRemoveListener(String JSFunctionName) {
+		app.getScriptManager().registerRemoveListener(JSFunctionName);
+	}
+	
+	public synchronized void unregisterRemoveListener(String JSFunctionName) {
+		app.getScriptManager().unregisterRemoveListener(JSFunctionName);
+	}
+	
+	public synchronized void registerClearListener(String JSFunctionName) {
+		app.getScriptManager().registerClearListener(JSFunctionName);
+	}
+
+	public synchronized void unregisterClearListener(String JSFunctionName) {
+		app.getScriptManager().unregisterClearListener(JSFunctionName);
+	}
+
+	public synchronized void registerRenameListener(String JSFunctionName) {
+		app.getScriptManager().registerRenameListener(JSFunctionName);
+	}
+	
+	public synchronized void unregisterRenameListener(String JSFunctionName) {
+		app.getScriptManager().unregisterRenameListener(JSFunctionName);
+	}
+	
+	public synchronized void registerUpdateListener(String JSFunctionName) {
+		app.getScriptManager().registerUpdateListener(JSFunctionName);
+	}
+	
+	public synchronized void unregisterUpdateListener(String JSFunctionName) {
+		app.getScriptManager().unregisterUpdateListener(JSFunctionName);
+	}
+
+	public synchronized void registerObjectUpdateListener(String objName, String JSFunctionName) {
+		app.getScriptManager().registerObjectUpdateListener(objName, JSFunctionName);
+	}
+	
+	public synchronized void unregisterObjectUpdateListener(String objName) {
+		app.getScriptManager().unregisterObjectUpdateListener(objName);
+	}
+
+	public boolean isMoveable(String objName) {
+		GeoElement geo = kernel.lookupLabel(objName);
+		if (geo == null) 
+			return false;
+		else
+			return geo.isMoveable();
+	}
+
+
+		
 	
           
 }// class GgbAPI
